/*  This extension adds some accelerator buttons to a Filepopup widget
    Copyright (C) 1997-1998  Richard Gooch
    This code is distributed under the GNU General Public License
*/
#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <X11/Intrinsic.h>
#include <X11/Xos.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Command.h>
#include <karma.h>
#include <karma_a.h>
#include <Xkw/Filepopup.h>
#include <Xkw/Filewin.h>


/*  Private functions  */
STATIC_FUNCTION (Widget add_button,
		 (Widget box, CONST char *label, CONST char *directory) );
STATIC_FUNCTION (void chdir_cbk, (Widget w, XtPointer client_data,
				  XtPointer call_data) );


/*  Public functions follow  */

flag WidgetInitialise (Widget Request, Widget New)
{
    Widget box;

    XtVaGetValues (New, XkwNboxWidget, &box, NULL);
    /*  Add the various personalised buttons  */
    add_button (box, "/", "/");
    add_button (box, "/data", "/data");
    add_button (box, "/cdrom", "/cdrom");
    return (TRUE);
}   /*  End Function WidgetInitialise  */


/*  Private functions follow */

static Widget add_button (Widget box, CONST char *label, CONST char *directory)
/*  [SUMMARY] Add a button to a box.
    <box> The enclosing box widget.
    <upper> The widget under which the new widget should be placed.
    <left> The widget to the left of where the widget should be placed.
    <label> The label to appear in the button. The pointer must remain valid.
    <directory> The directory to change to when the button is pressed. The
    pointer must remain valid.
    [RETURNS] The new button widget.
*/
{
    Widget w;

    w = XtVaCreateManagedWidget ("button", commandWidgetClass, box,
				 XtNlabel, label,
				 NULL);
    XtAddCallback (w, XtNcallback, chdir_cbk, (XtPointer) directory);
    return (w);
}   /*  End Function add_button  */

static void chdir_cbk (Widget w, XtPointer client_data, XtPointer call_data)
{
    Widget filewin, parent, filepopup;
    CONST char *dirname = (CONST char *) client_data;
    static char function_name[] = "Filepopup-ext_chdir_cbk";

    for ( parent = XtParent (w), filepopup = NULL;
	  (filepopup == NULL) && (parent != NULL);
	  parent = XtParent (parent) )
    {
	if ( XtIsFilepopup (parent) ) filepopup = parent;
    }
    if (filepopup == NULL)
    {
	fprintf (stderr, "Could not find a Filepopup ancestor\n");
	a_prog_bug (function_name);
    }
    XtVaGetValues (filepopup, XkwNfilewinWidget, &filewin, NULL);
    XkwFilewinChangeDirectory (filewin, dirname);
}   /*  End Function chdir_cbk  */
