/*-----------------------------------------------------------*/
/*  Communications sample program: instrument control server */
/*-----------------------------------------------------------*/

#include <stdio.h>
#include <karma_conn.h>
#include <karma_dm.h>
#include <karma_ch.h>
#include <karma_r.h>
#include <karma_e.h>


#define PORT_NUMBER 10000


STATIC_FUNCTION (flag periodic_func, (KPeriodicEventFunc func, void *info) );
STATIC_FUNCTION (flag read_func, (Connection connection, void **info) );


void main ()
{
    KPeriodicEventList event_list;
    unsigned int port_number = PORT_NUMBER;

    /*  Initialise communications package  */
    dm_native_setup ();
    conn_initialise ( ( void (*) () ) NULL );
    /*  Register server protocols  */
    conn_register_server_protocol ("periodic_data", 0, 0,
				   ( flag (*) () ) NULL, ( flag (*) () ) NULL,
				   ( void (*) () ) NULL);
    conn_register_server_protocol ("query_data", 0, 0,
				   ( flag (*) () ) NULL, read_func,
				   ( void (*) () ) NULL);
    /*  Allocate a port  */
    if ( !conn_become_server (&port_number, 0) )
    {
        fprintf (stderr, "Could not become a server\n");
        exit (1);
    }
    fprintf (stderr, "Port number: %u\n", port_number);
    event_list = e_unix_create_list (0, 1, NULL);
    e_register_func (event_list, periodic_func, NULL, 1, DISPATCH_SYNCHRONOUS);
    /*  Enter polling loop (forever)  */
    while (TRUE) dm_native_poll (-1);
}   /*  End Function main  */


/*  Private functions follow  */

static flag periodic_func (KPeriodicEventFunc func, void *info)
/*  [SUMMARY] Event callback.
    [PURPOSE] This routine is called when a periodic event occurs.
    <func> The KPeriodicEventFunc object.
    <info> A pointer to arbitrary information.
    [RETURNS] TRUE if the event function should be called again, else FALSE.
*/
{
    Connection conn;
    Channel ch;
    char dummy;
    unsigned int count, num_conn;

    num_conn = conn_get_num_serv_connections ("periodic_data");
    for (count = 0; count < num_conn; ++count)
    {
	conn = conn_get_serv_connection ("periodic_data", count);
	ch = conn_get_channel (conn);
	ch_write (ch, &dummy, 1);
	ch_flush (ch);
    }
    return (TRUE);
}   /*  End Function periodic_func  */

static flag read_func (Connection connection, void **info)
/*  [SUMMARY] Connection read event callback.
    [PURPOSE] This routine is called when data is ready to be read from a
    connection.
    <connection> The connection object.
    <info> A pointer to the arbitrary information pointer. This may be modified
    [RETURNS] TRUE on successful reading, else FALSE (indicating the connection
    should be closed).
    [NOTE] The <<close_func>> will be called if this routine returns FALSE.
*/
{
    Channel ch;
    char dummy;

    ch = conn_get_channel (connection);
    ch_read (ch, &dummy, 1);
    ch_write (ch, &dummy, 1);
    ch_flush (ch);
    return (TRUE);
}   /*  End Function read_func  */
