/*----------------------------------------------------------*/
/*  Communications sample program:  wait client             */
/*----------------------------------------------------------*/

#include <stdio.h>
#include <math.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/resource.h>
#include <karma_conn.h>
#include <karma_dm.h>
#include <karma_ch.h>
#include <karma_r.h>


#define PORT_NUMBER 10000


STATIC_FUNCTION (flag read_func, (Connection connection, void **info) );


void main (int argc, char **argv)
{
    /*  Initialise communications package  */
    dm_native_setup ();
    conn_initialise ( ( void (*) () ) NULL );
    /*  Register server protocols  */
    conn_register_client_protocol ("periodic_data", 0, 0,
                                   ( flag (*) () ) NULL, ( flag (*) () ) NULL,
				   read_func, ( void (*) () ) NULL);
    /*  Connect to server  */
    if ( !conn_attempt_connection (argv[1], PORT_NUMBER, "periodic_data") )
    {
        fprintf (stderr, "Could not connect to server\n");
        exit (1);
    }
    /*  Enter polling loop (forever)  */
    while (TRUE) dm_native_poll (-1);
}   /*  End Function main  */


/*  Private functions follow  */

static flag read_func (Connection connection, void **info)
/*  [SUMMARY] Connection read event callback.
    [PURPOSE] This routine is called when data is ready to be read from a
    connection.
    <connection> The connection object.
    <info> A pointer to the arbitrary information pointer. This may be modified
    [RETURNS] TRUE on successful reading, else FALSE (indicating the connection
    should be closed).
    [NOTE] The <<close_func>> will be called if this routine returns FALSE.
*/
{
    Channel ch;
    char dummy;
    double d_time;
    struct timeval curr_time;
    static double old_time = 0.0;
    static struct timezone tz = {0, 0};

    ch = conn_get_channel (connection);
    ch_read (ch, &dummy, 1);
    gettimeofday (&curr_time, &tz);
    d_time = curr_time.tv_sec;
    d_time += (double) curr_time.tv_usec * 1e-6;
    if (old_time < 1e-6) old_time = d_time;
    d_time -= old_time;
    fprintf (stderr, "%f s\t%f.2 us\n",
	     d_time, ( d_time - rint (d_time) ) * 1e6);
    return (TRUE);
}   /*  End Function read_func  */
