/*----------------------------------------------------------*/
/*  Parabola display sample programme                       */
/*----------------------------------------------------------*/

#include <stdio.h>
#include <unistd.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <karma.h>
#include <k_version.h>
#include <karma_overlay.h>
#include <karma_xtmisc.h>
#include <karma_im.h>
#include <Xkw/ImageDisplay.h>

#define VERSION "1.1"
#define NUM_SEGMENTS 100


STATIC_FUNCTION (void draw_parabola, (KOverlayList olist) );


/*  Private data  */
String fallback_resources[] =
{
    "Kparabola*Canvas*background:                        black",
    "Kparabola*Command*background:                       grey70",
    "Kparabola*Toggle*background:                        grey80",
    "Kparabola*ImageDisplay*quit*background:             orange",
    "Kparabola*background:                               aquamarine",
    "Kparabola*font:                                     9x15bold",
    NULL
};


int main (int argc, char **argv)
{
    KWorldCanvas wc_pseudo, wc_true;
    KViewableOverlayList vlist;
    KOverlayList olist;
    XtAppContext app_context;
    Widget main_shell, image_display;
    /*static char function_name[] = "main";*/

    /*  Initialise module  */
    im_register_module_name ("kparabola");
    im_register_module_version_date (VERSION);
    im_register_lib_version (KARMA_VERSION);
    /*  Start up Xt  */
    main_shell = xtmisc_init_app_initialise (&app_context, "Kparabola",
					     NULL, 0,
					     &argc, argv, fallback_resources,
					     XTMISC_INIT_ATT_COMMS_SETUP, TRUE,
					     XTMISC_INIT_ATT_END,
					     NULL);
    image_display = XtVaCreateManagedWidget ("topForm",
					     imageDisplayWidgetClass,
					     main_shell,
					     XtNborderWidth, 0,
					     XkwNnumDatasets, 0,
					     XkwNcreateFilepopup, FALSE,
					     NULL);
    XtRealizeWidget (main_shell);
    XtVaGetValues (image_display,
                   XkwNpseudoColourCanvas, &wc_pseudo,
                   XkwNtrueColourCanvas, &wc_true,
                   NULL);
    if (wc_pseudo == NULL) wc_pseudo = wc_true;
    XtVaSetValues (image_display, XkwNvisibleCanvas, wc_pseudo, NULL);
    olist = overlay_va_create_list (NULL, NULL,
				    OVERLAY_ATT_END);
    overlay_specify_canvas (olist, wc_pseudo);
    vlist = overlay_create_viewable (olist, wc_pseudo);
    overlay_set_active (vlist, TRUE, FALSE, TRUE, FALSE, 0);
    draw_parabola (olist);
    XtAppMainLoop (app_context);
    return (RV_OK);
}   /*  End Function main   */

static void draw_parabola (KOverlayList olist)
/*  [PURPOSE] This routine will draw a parabola into an overlay list.
    <olist> The overlay list.
    [RETURNS] Nothing.
*/
{
    int count;
    double x_factor, x;
    double x_arr[NUM_SEGMENTS];
    double y_arr[NUM_SEGMENTS];

    /*  Generate array of points  */
    x_factor = 2.0 / (double) (NUM_SEGMENTS);
    for (count = 0; count < NUM_SEGMENTS; ++count)
    {
        x = (double) (count - NUM_SEGMENTS / 2) * x_factor;
        x_arr[count] = x / 2.0 + 0.5;
        y_arr[count] = x * x;
    }
    /*  Draw lines  */
    overlay_lines (olist, NUM_SEGMENTS, NULL, x_arr, y_arr, "yellow");
}   /*  End Function draw_parabola  */
