/*  compress-countlog.c

    Source file for  compress-countlog  (compress log of counts).

    Copyright (C) 1999  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*  This Karma utility programme will compress a logfile containing counts of
    some quantity(ies), such as lines of code.


    Written by      Richard Gooch   9-FEB-1999

    Updated by      Richard Gooch   10-FEB-1999

    Last updated by Richard Gooch   4-MAR-1999: Prevent writing last line twice


*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <time.h>
#include <errno.h>
#include <karma.h>
#include <karma_chs.h>
#include <karma_ch.h>
#include <karma_st.h>


int main (int argc, char **argv)
{
    Channel ch_in, ch_out;
    flag prev_printed;
    char *filename_in, *ptr, *prev_ptr;
    char filename_out[STRING_LENGTH], filename_backup[STRING_LENGTH];
    char buffer[STRING_LENGTH], prev_buf[STRING_LENGTH];
    static char *usage = "compress-countlog logfile";

    if (argc != 2)
    {
	fprintf (stderr, "Usage:\t%s\n", usage);
	exit (RV_MISSING_PARAM);
    }
    filename_in = argv[1];
    if ( ( ch_in = ch_open_file (filename_in, "r") ) == NULL )
    {
	fprintf (stderr, "Error opening file: \"%s\"\t%s\n",
		 filename_in, ERRSTRING);
	exit (RV_CANNOT_OPEN);
    }
    sprintf (filename_out, "%s~~", filename_in);
    if ( ( ch_out = ch_open_file (filename_out, "w") ) == NULL )
    {
	fprintf (stderr, "Error opening file: \"%s\"\t%s\n",
		 filename_out, ERRSTRING);
	exit (RV_CANNOT_OPEN);
    }
    prev_buf[0] = '\0';
    prev_ptr = prev_buf;
    prev_printed = TRUE;
    while ( ch_getl (ch_in, buffer, STRING_LENGTH) )
    {
	if ( ( ptr = strchr (buffer, '*') ) == NULL )
	{
	    fprintf (stderr, "No '*' found in line: \"%s\"\n", buffer);
	    unlink (filename_out);
	    exit (RV_BAD_DATA);
	}
	++ptr;
	if (strcmp (ptr, prev_ptr) == 0) prev_printed = FALSE;
	else
	{
	    if (!prev_printed) ch_puts (ch_out, prev_buf, TRUE);
	    ch_puts (ch_out, buffer, TRUE);
	    prev_printed = TRUE;
	}
	strcpy (prev_buf, buffer);
	prev_ptr = prev_buf + (ptr - buffer);
    }
    if (!prev_printed) ch_puts (ch_out, prev_buf, TRUE);
    ch_close (ch_in);
    ch_close (ch_out);
    sprintf (filename_backup, "%s~", filename_in);
    if (rename (filename_in, filename_backup) != 0)
    {
	fprintf (stderr, "Error renaming file\t%s\n", ERRSTRING);
	unlink (filename_out);
	exit (RV_SYS_ERROR);
    }
    rename (filename_out, filename_in);
    return (0);
}   /*  End Function main  */
