/*  This programme will print all Karma packages which have changed since the
  documentation was last built.


    Written by      Richard Gooch   26-NOV-1994

    Updated by      Richard Gooch   26-NOV-1994

    Updated by      Richard Gooch   7-DEC-1994: Stripped declaration of  errno

    Updated by      Richard Gooch   28-MAR-1997: Pass in doc directory.

    Last updated by Richard Gooch   15-AUG-1999: Made egcs happy.


*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <math.h>
#include <karma.h>
#include <karma_dir.h>
#include <karma_ch.h>
#include <karma_st.h>
#include <karma_r.h>
#include <karma_m.h>
#include <karma_a.h>


STATIC_FUNCTION (flag package_changed, (char *package_name) );


/*  Private data follows  */
static char *doc_dirname = NULL;


int main (int argc, char *argv[])
{
    KDir dir;
    KFileInfo *file;
    static char usage_string[]="Usage:\tfind_changed_packages doc_directory\n";
    /*static char function_name[] = "main";*/

    if (argc != 2)
    {
	fprintf (stderr, usage_string);
	exit (RV_TOO_MANY_PARAM);
    }
    doc_dirname = argv[1];
    /*  Start to go through source directory  */
    if ( ( dir = dir_open (".") ) == NULL )
    {
	fprintf (stderr, "Error opening directory\t%s\n", ERRSTRING);
	exit (RV_SYS_ERROR);
    }
    while ( ( file = dir_read (dir, KDIR_NO_DOTS) ) != NULL )
    {
	if (file->type != KFILETYPE_DIRECTORY) continue;
	if ( package_changed ( file->filename ) )
	{
	    printf ("%s ", file->filename);
	}
    }
    dir_close (dir);
    printf ("\n");
    fflush (stdout);
    return (0);
}   /*  End Function main  */

static flag package_changed (char *package_name)
/*  This routine will determine if a package has changed since it's
    documentation was last built.
    The package name must be given by  package_name  .
    The routine returns TRUE if the package changed, else it returns FALSE.
*/
{
    KDir dir;
    KFileInfo *file;
    struct stat statbuf;
    char filename[STRING_LENGTH];
    extern char *doc_dirname;

    /*  Construct documentation file name  */
    sprintf (filename, "%s/%s.doc", doc_dirname, package_name);
    /*  Get stats on file  */
    if (stat (filename, &statbuf) != 0)
    {
	if (errno == ENOENT) return (TRUE);
	fprintf (stderr, "Error getting stats on file: \"%s\"\t%s\n",
		 filename, ERRSTRING);
	exit (RV_SYS_ERROR);
    }
    if ( ( dir = dir_open (package_name) ) == NULL )
    {
	fprintf (stderr, "Error opening directory\t%s\n", ERRSTRING);
	exit (RV_SYS_ERROR);
    }
    while ( ( file = dir_read (dir, KDIR_NO_DOTS) ) != NULL )
    {
	if ( file->type != KFILETYPE_REGULAR ) continue;
	if (statbuf.st_mtime > file->mtime) continue;
	dir_close (dir);
	return (TRUE);
    }
    dir_close (dir);
    return (FALSE);
}   /*  End Function package_changed  */
