/*  This programme will print all Karma widgets which have changed since the
  documentation was last built.


    Written by      Richard Gooch   10-MAY-1997

    Updated by      Richard Gooch   10-MAY-1997: Copied from
  find_changed_packages.c

    Last updated by Richard Gooch   15-AUG-1999: Made egcs happy.


*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <math.h>
#include <karma.h>
#include <karma_dir.h>
#include <karma_ch.h>
#include <karma_st.h>
#include <karma_r.h>
#include <karma_m.h>
#include <karma_a.h>


STATIC_FUNCTION (flag widget_changed,
		 (CONST char *doc_dirname, KFileInfo *file) );


/*  Private data follows  */


int main (int argc, char *argv[])
{
    KDir dir;
    KFileInfo *file;
    char *doc_dirname;
    static char usage_string[]="Usage:\tfind_changed_widgets doc_directory\n";
    /*static char function_name[] = "main";*/

    if (argc != 2)
    {
	fprintf (stderr, usage_string);
	exit (RV_TOO_MANY_PARAM);
    }
    doc_dirname = argv[1];
    /*  Start to go through source directory  */
    if ( ( dir = dir_open (".") ) == NULL )
    {
	fprintf (stderr, "Error opening directory\t%s\n", ERRSTRING);
	exit (RV_SYS_ERROR);
    }
    while ( ( file = dir_read (dir, KDIR_NO_DOTS) ) != NULL )
    {
	if (file->type != KFILETYPE_REGULAR) continue;
	if ( widget_changed (doc_dirname, file) )
	    printf ("%s\n", file->filename);
    }
    dir_close (dir);
    printf ("\n");
    fflush (stdout);
    return (0);
}   /*  End Function main  */

static flag widget_changed (CONST char *doc_dirname, KFileInfo *file)
/*  [SUMMARY] Test if a widget has changed.
    <doc_dirname> The directory name of the documentation directory.
    <file> The source file information.
    [RETURNS] TRUE if the widget has changed, else FALSE.
*/
{
    struct stat doc_statbuf, statbuf;
    char *ptr;
    char name[STRING_LENGTH], txt[STRING_LENGTH];

    strcpy (name, file->filename);
    if ( ( ptr = strrchr (name, '.') ) == NULL )
    {
	fprintf (stderr, "Bad filename: \"%s\"\n", file->filename);
	exit (RV_BAD_DATA);
    }
    if (strcmp (ptr, ".c") != 0)
    {
	fprintf (stderr, "Bad filename: \"%s\"\n", file->filename);
	exit (RV_BAD_DATA);
    }
    *ptr = '\0';
    sprintf (txt, "doc/%s.header", name);
    if (stat (txt, &statbuf) != 0)
    {
	if (errno == ENOENT)
	{
	    fprintf (stderr, "%s does not exist: WRITE IT\n", txt);
	    exit (RV_BAD_DATA);
	}
	fprintf (stderr, "Error getting stats on file: \"%s\"\t%s\n",
		 txt, ERRSTRING);
	exit (RV_SYS_ERROR);
    }
    sprintf (txt, "%s/%s.doc", doc_dirname, name);
    if (stat (txt, &doc_statbuf) != 0)
    {
	if (errno == ENOENT) return (TRUE);
	fprintf (stderr, "Error getting stats on file: \"%s\"\t%s\n",
		 txt, ERRSTRING);
	exit (RV_SYS_ERROR);
    }
    if (doc_statbuf.st_mtime < file->mtime) return (TRUE);
    if (doc_statbuf.st_mtime < statbuf.st_mtime) return (TRUE);
    return (FALSE);
}   /*  End Function widget_changed  */
