/*  latex-words.c

    Source file for latex-words

    Copyright (C) 1999  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*
    This programme will count the number of words in a LaTeX file. It excludes
    LaTeX comments, commands and verbatim sections.


    Written by      Richard Gooch   25-FEB-1999

    Last updated by Richard Gooch   26-JUL-1999: Added test for "\\bolditem".


*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <karma.h>
#include <karma_chs.h>
#include <karma_ch.h>


/*  Private functions  */
STATIC_FUNCTION (char *skip_whitespace, (char *ptr) );
STATIC_FUNCTION (char *skip_nonwhitespace, (char *ptr) );


/*  Public functions follow  */

int main (int argc, char **argv)
{
    flag verbatim = FALSE;
    unsigned int word_count = 0;
    char *ptr;
    char line[STRING_LENGTH];
    /*static char usage_string[] = "Usage:\tlatex-words";*/

    ch_open_stdin ();
    while ( chs_get_line2 (ch_stdin, line, STRING_LENGTH, '\0') )
    {
	if ( verbatim && (strcmp (line, "\\end{verbatim}") == 0) )
	{
	    verbatim = FALSE;
	    continue;
	}
	if (verbatim) continue;
	if (strcmp (line, "\\begin{verbatim}") == 0)
	{
	    verbatim = TRUE;
	    continue;
	}
	if (line[0] == '%') continue;
	ptr = line;
	if (ptr[0] == '\\')
	{
	    if (strncmp (ptr, "\\item", 5) == 0) ptr = skip_nonwhitespace(ptr);
	    else if (strncmp (ptr, "\\name", 5) == 0) ++ptr;  /* Dummy-op  */
	    else if (strncmp (ptr, "\\bolditem", 9) == 0)
		ptr = strchr (ptr, '{');
	    else continue;
	}
	if (ptr == NULL) continue;
	while (*ptr != '\0')
	{
	    if ( !isspace (*ptr) )
	    {
		++word_count;
		ptr = skip_nonwhitespace (ptr);
	    }
	    ptr = skip_whitespace (ptr);
	}
    }
    printf ("%u\n", word_count);
    fflush (stdout);
    return (RV_OK);
}   /*  End Function main  */


/*  Private functions follow  */

static char *skip_whitespace (char *ptr)
/*  [SUMMARY] Skip whitespace.
    <ptr> A pointer to a string.
    [RETURNS] A pointer to the first non-whitespace character.
*/
{
    while ( isspace (*ptr) ) ++ptr;
    return (ptr);
}   /*  End Function skip_whitespace  */

static char *skip_nonwhitespace (char *ptr)
/*  [SUMMARY] Skip non-whitespace.
    <ptr> A pointer to a string.
    [RETURNS] A pointer to the first whitespace character.
*/
{
    while ( (*ptr != '\0') && !isspace (*ptr) ) ++ptr;
    return (ptr);
}   /*  End Function skip_nonwhitespace  */
