/*  time-kread.c

    Source file for  time-kread  (time Karma reading routines).

    Copyright (C) 1999  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*  This Karma utility programme will time how long it takes to read a file
    using the Karma data reading functions.


    Written by      Richard Gooch   18-AUG-1999

    Last updated by Richard Gooch   18-AUG-1999


*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/time.h>
#include <errno.h>
#include <karma.h>
#include <karma_foreign.h>
#include <karma_ch.h>


int main (int argc, char **argv)
{
    signed long time_taken;
    struct timeval start_time, stop_time;
    multi_array *multi_desc;
    char *filename;
    static char *usage = "time-kread file";
    static char function_name[] = "main";

    if (argc != 2)
    {
	fprintf (stderr, "Usage:\t%s\n", usage);
	exit (RV_MISSING_PARAM);
    }
    filename = argv[1];
    ch_map_disc (filename, K_CH_MAP_LOCAL, FALSE, FALSE);
    if (gettimeofday (&start_time, NULL) != 0)
    {
	fprintf (stderr, "Error getting time\t%s\n", ERRSTRING);
	exit (RV_SYS_ERROR);
    }
    if ( ( multi_desc = foreign_guess_and_read (filename, K_CH_MAP_LOCAL,
						FALSE, NULL,
						FA_GUESS_READ_END) )
	 == NULL )
    {
	fprintf (stderr, "%s: error reading file: \"%s\"\n",
		 function_name, filename);
	return (RV_UNDEF_ERROR);
    }
    if (gettimeofday (&stop_time, NULL) != 0)
    {
	fprintf (stderr, "Error getting time\t%s\n", ERRSTRING);
	exit (RV_SYS_ERROR);
    }
    time_taken = (stop_time.tv_usec - start_time.tv_usec) / 1000;
    time_taken += (stop_time.tv_sec - start_time.tv_sec) * 1000;
    fprintf (stderr, "Time taken: %ld milliseconds\n", time_taken);
    return (0);
}   /*  End Function main  */
