/*LINTLIBRARY*/
/*  atm.c
    This code implements ATM stub functions.

    Copyright (C) 1998  Richard Gooch

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/
/*

    This file contains stub functions for ATM.


    Written by      Richard Gooch   13-DEC-1998

    Last updated by Richard Gooch   25-DEC-1998: Created read, write and close
  stub functions.


*/
#include <stdio.h>
#include <karma.h>
#include <karma_r.h>
#include <os.h>

#ifdef HAS_ATM

struct unused
{
    int a;
};

#else /*  HAS_ATM  */


/*  Private functions  */
STATIC_FUNCTION (void print_error, (CONST char *function_name) );


/*  Public functions follow  */

/*STUB_FUNCTION*/
int r_atm_alloc (char *addr_string, unsigned *addr_len)
/*  [SUMMARY] Create an ATMSVC socket and bind to a random port number.
    <addr_string> The textual representation of the ATM address is written here
    <addr_len> The length of the address string. This is updated.
    [NOTE] The close-on-exec flags of the socket is set such that the socket
    will close on a call to execve(2V).
    [RETURNS] The file descriptor of the socket on success, else -1.
*/
{
    print_error ("r_atm_alloc");
    return (-1);
}   /*  End Function r_atm_alloc  */

/*STUB_FUNCTION*/
int r_atm_connect (CONST char *addr_string, unsigned int addr_len,
		   struct karma_qos *qos)
/*  [SUMMARY] Make a connection using ATM SVC with AAL5.
    <addr_string> The textual representation of the ATM address to connect to.
    <addr_len> The length of the address string.
    <qos> The Quality of Service parameters. This will be updated to reflect
    the parameters provided. This may be NULL.
    [NOTE] The close-on-exec flags of the socket is set such that the socket
    will close on a call to execve(2V).
    [NOTE] This function is not implemented on platforms without ATM support.
    [RETURNS] The file descriptor of the opened connection on success, else -1.
*/
{
    print_error ("r_atm_connect");
    return (-1);
}   /*  End Function r_atm_connect  */

/*STUB_FUNCTION*/
int r_atm_accept (int fd, struct karma_qos *qos)
/*  [SUMMARY] Accept a connection on an ATM dock.
    <fd> The connected socket.
    <qos> The QoS parameters will be written here.
    [RETURNS] The file descriptor of the opened connection on success, else -1.
*/
{
    print_error ("r_atm_accept");
    return (-1);
}   /*  End Function r_atm_accept  */

/*STUB_FUNCTION*/
iaddr r_atm_read (int fd, char *buf, uaddr nbytes)
/*  [SUMMARY] Read bytes from a file descriptor.
    [PURPOSE] This routine is similar to the system <<read(2)>> call, except
    that interrupted system calls are restarted.
    <fd> The file descriptor to read from. This descriptor must NOT be set to
    non-blocking IO.
    <buf> The buffer in which to write the data.
    <nbytes> The number of bytes to read.
    [RETURNS] The number of bytes requested on success, the number of bytes
    read on end of file (or closure) and -1 on error.
*/
{
    print_error ("r_atm_read");
    return (-1);
}   /*  End Function r_atm_read  */

/*STUB_FUNCTION*/
iaddr r_atm_write (int fd, CONST char *buf, uaddr nbytes)
/*  [SUMMARY] Write bytes to a file descriptor.
    <fd> The file descriptor to write to. This descriptor must NOT be set to
    non-blocking IO.
    <buf> The buffer in which to write the data.
    <nbytes> The number of bytes to write.
    [RETURNS] The number of bytes requested on success, else -1 indicating
    error.
*/
{
    print_error ("r_atm_write");
    return (-1);
}   /*  End Function r_atm_write  */

/*STUB_FUNCTION*/
flag r_atm_close (int fd)
/*  [SUMMARY] Close a file descriptor.
    <fd> The file descriptor to close.
    [RETURNS] TRUE on success, else FALSE.
*/
{
    print_error ("r_atm_close");
    return (FALSE);
}   /*  End Function r_atm_close  */


/*  Private functions follow  */

static void print_error (CONST char *function_name)
{
    fprintf (stderr, "%s: no ATM support\n", function_name);
}   /*  End Function print_error  */

#endif /*  !HAS_ATM  */
