/*  main_xt.c

    Main file for  MultibeamView  (X11 image/movie display tool for Karma).

    Copyright (C) 1997-2003  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*
    This Karma module will enable on-screen imaging.
    This module runs on an X11 server.


    Written by      Richard Gooch   28-JAN-1997: Copied from kview/main_xt.c

    Updated by      Richard Gooch   29-JAN-1997

    Updated by      Richard Gooch   2-FEB-1997: Added "filename" association to
  dataset(s).

    Updated by      Richard Gooch   12-FEB-1997: Only use GLISH "Dataset1"
  events.

    Updated by      Richard Gooch   25-FEB-1997: Fixed bug when associating
  filename with dataset: check for PseudoColour or TrueColour dataset.

    Updated by      Richard Gooch   9-MAR-1997: Took account of change to
  <xtmisc_init_app_initialise>.

    Updated by      Richard Gooch   18-MAR-1997: Added display of scrolling
  RA&DEC information.

    Updated by      Richard Gooch   19-APR-1997: Now use internal file loader
  in Filewinpopup widget.

    Updated by      Richard Gooch   22-JUN-1997: Took account of change to
  <xtmisc_init_app_initialise> to double varargs list.

    Updated by      Richard Gooch   4-AUG-1997: Set resources for middle and
  right mouse button zooming.

    Updated by      Richard Gooch   10-AUG-1997: Created -create_at_init switch

    Updated by      Richard Gooch   12-AUG-1997: No longer initialise <contour>
  package because ImageDisplay widget does it.

    Updated by      Richard Gooch   7-DEC-1997: Added font resource for new
  intensity menu and bumped minor version number because panner implemented.

    Updated by      Richard Gooch   1-JAN-1998: Made use of new XkwNwmtitle
  resource.

    Updated by      Richard Gooch   1-MAR-1998: Made use of new filewinWidget
  resource.

    Updated by      Richard Gooch   9-MAY-1998: No longer attach filename
  object.

    Updated by      Richard Gooch   17-MAY-1998: Trap illegal values.

    Updated by      Richard Gooch   2-SEP-1998: Removed overlay code.

    Updated by      Richard Gooch   5-SEP-1998: Used new comms. setup.

    Updated by      Richard Gooch   17-SEP-1998: Use SimpleMenu class name in
  resources.

    Updated by      Richard Gooch   29-OCT-1998: Added binding of 'v' key.

    Updated by      Richard Gooch   5-JUL-1999: Added "-quiet" option.

    Updated by      Richard Gooch   30-AUG-1999: Added dirname font resource.

    Updated by      Richard Gooch   21-FEB-2000: Removed unnecessary extern
  declarations.

    Updated by      Richard Gooch   11-MAY-2001: Changed dirname font resource.

    Updated by      Richard Gooch   3-OCT-2001: Added translations for Filewin.

    Updated by      Richard Gooch   21-DEC-2001: Included k_version.h directly.

    Updated by      Richard Gooch   5-DEC-2002: Took account of resource name
  change for ImageDisplay widget.

    Last updated by Richard Gooch   4-OCT-2003: Added #include <karma_iarray.h>


*/
#include <stdio.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Toggle.h>
#include <karma.h>
#include <k_version.h>
#define NEW_WIN_SCALE
#define X11
#include <k_event_codes.h>
#include <karma_viewimg.h>
#include <karma_iarray.h>
#include <karma_xtmisc.h>
#include <karma_dsxfr.h>
#include <karma_conn.h>
#include <karma_dir.h>
#include <karma_gls.h>
#include <karma_wcs.h>
#include <karma_ds.h>
#include <karma_im.h>
#include <karma_hi.h>
#include <karma_ic.h>
#include <karma_st.h>
#include <karma_a.h>
#include <karma_m.h>
#include <Xkw/ImageDisplay.h>
#include <Xkw/Filewin.h>
#include <Xkw/View2Datasets.h>
#include "kview.h"
#include "MultibeamView.h"


#define VERSION "1.1.15"


/*  Private functions  */
STATIC_FUNCTION (void register_handlers, (KWorldCanvas canvas, Widget w) );
STATIC_FUNCTION (flag data_event_func,
		 (void *object, multi_array *multi_desc,
		  CONST char *domain, CONST char *name) );
STATIC_FUNCTION (void radec_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (flag position_func,
		 (KWorldCanvas canvas, double x, double y,
		  unsigned int event_code, void *e_info,
		  void **f_info, double x_lin, double y_lin,
		  int x_pix, int y_pix) );


/*  Private data  */
String fallback_resources[] =
{
    "MultibeamView*pseudoColourCanvas*background:                 black",
    "MultibeamView*directColourCanvas*background:                 black",
    "MultibeamView*trueColourCanvas*background:                   black",
    "MultibeamView*pseudoColourStereoCanvas*background:           black",
    "MultibeamView*directColourStereoCanvas*background:           black",
    "MultibeamView*trueColourStereoCanvas*background:             black",
    "MultibeamView*AnimateControl*positionCanvas.background:      black",
    "MultibeamView*pseudoColourCanvas*foreground:                 white",
    "MultibeamView*Command*background:                            grey70",
    "MultibeamView*Repeater*background:                           grey70",
    "MultibeamView*Ktoggle*background:                            grey80",
    "MultibeamView*closeButton*background:                        grey90",
    "MultibeamView*ChoiceMenu.background:                         turquoise",
    "MultibeamView*ExclusiveMenu.background:                      turquoise",
    "MultibeamView*Value*background:                              #d0a0a0",
    "MultibeamView*ImageDisplay*quit*background:                  orange",
    "MultibeamView*ScrollableList.highlightPixel:                 Sea Green",
    "MultibeamView*SimpleSlider.foreground:                       Sea Green",
    "MultibeamView*SimpleProgressMeter.background:                turquoise3",
    "MultibeamView*SimpleProgressMeter.foreground:                grey60",
    "MultibeamView*ImageDisplay*trackLabel0*font:                 8x13bold",
    "MultibeamView*ImageDisplay*trackLabel1*font:                 8x13bold",
    "MultibeamView*ImageDisplay*trackLabel2*font:                 8x13bold",
    "MultibeamView*ImageDisplay*trackLabel3*font:                 8x13bold",
    "MultibeamView*SimpleMenu*font:                               10x20",
    "MultibeamView*ImageDisplay.Command.font:                     7x13bold",
    "MultibeamView*ImageDisplay.applicationBox.Command.font:      7x13bold",
    "MultibeamView*ImageDisplay.Ktoggle.font:                     7x13bold",
    "MultibeamView*ImageDisplay.ChoiceMenu.font:                  7x13bold",
    "MultibeamView*ImageDisplay.exportMenu.menuButton.font:       7x13bold",
    "MultibeamView*ImageDisplay.overlayMenu.menuButton.font:      7x13bold",
    "MultibeamView*ImageDisplay*zoomMenu*Unzoom*foreground:       red",
    "MultibeamView*SimpleSlider.borderWidth:                      0",
    "MultibeamView*IntensityPolicy*showIntensityReset:            True",
    "MultibeamView*IntensityPolicy*resetIntensityToggle.state:    True",
    "MultibeamView*IntensityPolicy*autoIntensityScale:            False",
    "MultibeamView*ZoomPolicy*showMiddleZoom:                     True",
    "MultibeamView*ZoomPolicy*middleZoomToggle.state:             True",
    "MultibeamView*ZoomPolicy*showRightZoom:                      True",
    "MultibeamView*ZoomPolicy*rightZoomToggle.state:              True",
    "MultibeamView*Filewin.dirname.textSink.font:                 6x13",
    "MultibeamView*font:                                          9x15bold",
    "MultibeamView*borderColor:                                   black",
    "MultibeamView*background:                                    aquamarine",
    "MultibeamView*foreground:                                    black",
    NULL
};
static XrmOptionDescRec Options[] =
{
    {"-private_cmap", ".topForm.multiCanvas.pseudoColourCanvas.forceNewCmap",
     XrmoptionNoArg, (XPointer) "True"},
    {"-num_colours", ".topForm.cmapSize", XrmoptionSepArg, (XPointer) NULL},
    {"-cmap_master", ".topForm.cmapMaster", XrmoptionSepArg, (XPointer) NULL},
    {"-fullscreen", ".topForm.fullscreen", XrmoptionNoArg, (XPointer) "True"},
    {"-create_at_init", "*createAtInit", XrmoptionNoArg, (XPointer) "True"},
    {"-verbose", "*verbose", XrmoptionNoArg, (XPointer) "True"},
};
static Widget main_shell = NULL;
static Widget image_display = NULL;
static Widget twinviewpopup = NULL;


int main (int argc, char **argv)
{
    KWorldCanvas wc_pseudo, wc_direct, wc_true;
    int argcount;
    XtAppContext app_context;
    Widget filewin, filepopup, w;
    extern flag verbose, quiet;
    extern char title_name[STRING_LENGTH];
    /*static char function_name[] = "main";*/

    /*  Initialise module  */
    im_register_module_name ("MultibeamView");
    im_register_module_version_date (VERSION);
    im_register_lib_version (KARMA_VERSION);
    /*  Start up Xt  */
    main_shell = xtmisc_init_app_initialise (&app_context, "MultibeamView",
					     Options, XtNumber (Options),
					     &argc, argv, fallback_resources,
					     XTMISC_INIT_ATT_MIN_CCELLS, 100,
					     XTMISC_INIT_ATT_COMMS_SETUP, TRUE,
					     XTMISC_INIT_ATT_VERBOSE, &verbose,
					     XTMISC_INIT_ATT_BECOME_SERVER,
					     TRUE,
					     XTMISC_INIT_ATT_TITLE, title_name,
					     XTMISC_INIT_ATT_END,
					     NULL);
    dsxfr_register_connection_limits (1, -1);
    xtmisc_set_icon (main_shell, ic_write_kimage_icon);
    image_display = XtVaCreateManagedWidget ("topForm",
					     imageDisplayWidgetClass,
					     main_shell,
					     XtNborderWidth, 0,
					     XkwNviewDatasets, TRUE,
					     XkwNnumDatasets, 2,
					     XkwNnumTrackLabels, 4,
					     NULL);
    if ( ( w = XtNameToWidget (image_display, "filesButton") ) != NULL )
    {
	/*  Force main "Files" button to be re-labelled as "Set1"  */
	XtVaSetValues (w,
		       XtNlabel, "Set1",
		       NULL);
    }
    filepopup = XtNameToWidget (image_display, "filewinPopup");
    XtVaGetValues (filepopup, XkwNfilewinWidget, &filewin, NULL);
    XtVaSetValues (filepopup,
		   XkwNfilenameTester, XkwFilewinStandardFileTester_nD,
		   XkwNdomainName, "FILE",
		   NULL);
    XtVaSetValues (filewin,
		   XkwNforwardSyntheticEvents, True,
		   XkwNtrapDirectoryDatasets, True,
		   NULL);
    ds_event_register_func (data_event_func, NULL);
    twinviewpopup = XtNameToWidget (image_display, "twinviewpopup");
    XtAddCallback (twinviewpopup, XkwNcursorCallback, radec_cbk, NULL);
    XtVaSetValues (twinviewpopup,
		   XkwNwmtitle, title_name,
		   NULL);
    XtRealizeWidget (main_shell);
    XtVaGetValues (image_display,
		   XkwNpseudoColourCanvas, &wc_pseudo,
		   XkwNdirectColourCanvas, &wc_direct,
		   XkwNtrueColourCanvas, &wc_true,
		   NULL);
    register_handlers (wc_pseudo, twinviewpopup);
    register_handlers (wc_direct, twinviewpopup);
    register_handlers (wc_true, twinviewpopup);
    gls_register_glish_event_handler (scroll_init_cbk, "scrollBufBegin", NULL);
    gls_register_glish_event_handler (scroll_add_cbk,"scrollBufAddImage",NULL);
    /*  Option processing after widgets created and realised  */
    for (argcount = 1; argcount < argc; ++argcount)
    {
	if (strcmp (argv[argcount], "-quiet") == 0) quiet = TRUE;
	else fprintf (stderr, "Option: \"%s\" ignored\n", argv[argcount]);
    }
    XtAppMainLoop (app_context);
    return (RV_OK);
}   /*  End Function main   */

void new_scroll_array ()
/*  [SUMMARY] Process a new scroll array.
    [RETURNS] Nothing.
*/
{
    extern Dataset set1;

    XkwView2DatasetsRegisterDataset (twinviewpopup, NULL, 0.0, 0.0,
				     NULL, NULL, NULL, 1);
    XkwView2DatasetsRegisterDataset (twinviewpopup,
				     set1.array_pseudo, set1.min, set1.max,
				     set1.array_red, set1.array_green,
				     set1.array_blue, 1);
}   /*  End Function new_scroll_array  */

void updated_scroll_array (long plane_index)
/*  [SUMMARY] Register a change in the scroll array.
    <plane_index> The index of the plane which changed. If this is less than
    0, the entire array changed.
    [RETURNS] Nothing.
*/
{
    if (plane_index < 0) XkwView2DatasetsInvalidateAll (twinviewpopup, 1);
    else XkwView2DatasetsInvalidatePlane (twinviewpopup, 1, 0, plane_index);
}   /*  End Function updated_scroll_array  */


/*  Private functions follow  */

static void register_handlers (KWorldCanvas canvas, Widget w)
/*  [SUMMARY] Register handlers for a canvas.
    <canvas> The canvas. This may be NULL.
    <w> The view control widget.
    [RETURNS] Nothing.
*/
{
    if (canvas == NULL) return;
    viewimg_register_position_event_func
	(canvas, ( flag (*)() )XkwView2DatasetsCanvasEventHandler, (void *) w);
    canvas_register_position_event_func (canvas, ( flag (*)() ) position_func,
					 (void *) w);
}   /*  End Function register_handlers  */

static flag data_event_func (void *object, multi_array *multi_desc,
			     CONST char *domain, CONST char *name)
/*  [SUMMARY] Process an event.
    <object> The object information pointer.
    <multi_desc> The multi_array descriptor.
    <domain> The domain from where the event was generated.
    <name> The name of the event in its domain.
    [RETURNS] TRUE if further callbacks should not be called, else FALSE.
*/
{
    Dataset *set = NULL;
    unsigned int set_number = 0;
    char *ptr;
    char filename[STRING_LENGTH], stripped_filename[STRING_LENGTH];
    extern Dataset set0, set1;

    if ( (strcmp (domain, "GLISH") == 0) && (strcmp (name, "Dataset1") != 0) )
	return (FALSE);
    if (strcmp (domain, "FILE") == 0) filename[0] = '\0';
    else
    {
	strcpy (filename, domain);
	strcat (filename, "::");
    }
    strcat (filename, name);
    strcpy (stripped_filename, filename);
    if ( ( ptr = strrchr (stripped_filename, '.') ) != NULL )
    {
	if (strcmp (ptr, ".kf") == 0) *ptr = '\0';
    }
    set = (set_number == 0) ? &set0 : &set1;
    XkwView2DatasetsRegisterDataset (twinviewpopup, NULL, 0.0, 0.0,
				     NULL, NULL, NULL, set_number);
    if ( !process_set (multi_desc, set) ) return (FALSE);
    XtVaSetValues (image_display,
		   XkwNimageName, stripped_filename,
		   NULL);
    XkwView2DatasetsRegisterDataset (twinviewpopup,
				     set->array_pseudo, set->min, set->max,
				     set->array_red, set->array_green,
				     set->array_blue, set_number);
    return (TRUE);
}   /*  End Function data_event_func  */

static void radec_cbk (Widget w, XtPointer client_data, XtPointer call_data)
{
    XkwView2DatasetsCallbackData cursor =
	(XkwView2DatasetsCallbackData) call_data;
    double ra, dec;
    Widget label;
    char string[STRING_LENGTH], ra_str[STRING_LENGTH], dec_str[STRING_LENGTH];
    extern iarray ra_arr, dec_arr;
    extern Widget image_display;

    if ( (ra_arr == NULL) || (dec_arr == NULL) ) return;
    if (cursor->set_number != 1) return;
    if ( ( cursor->pixel.y >= iarray_dim_length (ra_arr, 1) ) ||
	 ( cursor->pixel.z >= iarray_dim_length (ra_arr, 0) ) ) return;
    ra = D2 (ra_arr, (int) cursor->pixel.z, (int) cursor->pixel.y);
    dec = D2 (dec_arr, (int) cursor->pixel.z, (int) cursor->pixel.y);
    if ( DOUBLE_IS_BLANK (ra) || DOUBLE_IS_BLANK (dec) ) string[0] = '\0';
    else
    {
	wcs_astro_format_hms (ra_str, ra / 15.0);
	wcs_astro_format_dms (dec_str, dec);
	sprintf (string, "Ra: %s  Dec: %s", ra_str, dec_str);
    }
    label = XtNameToWidget (image_display, "trackLabel3");
    XtVaSetValues (label, XtNlabel, string, NULL);
}   /*  End Function radec_cbk  */

static flag position_func (KWorldCanvas canvas, double x, double y,
			   unsigned int event_code, void *e_info,
			   void **f_info, double x_lin, double y_lin,
			   int x_pix, int y_pix)
/*  [SUMMARY] Position event callback.
    [PURPOSE] This routine is a position event consumer for a world canvas.
    <canvas> The canvas on which the event occurred.
    <x> The horizontal world co-ordinate of the event.
    <y> The vertical world co-ordinate of the event.
    <event_code> The arbitrary event code.
    <e_info> A pointer to arbitrary event information.
    <f_info> A pointer to an arbitrary function information pointer.
    <x_lin> The horizontal linear world co-ordinate prior to the transform
    function being called.
    <y_lin> The vertical linear world co-ordinate prior to the transform
    function being called.
    <x_pix> The horizontal pixel co-ordinate.
    <y_pix> The vertical pixel co-ordinate.
    [NOTE] The world co-ordinate values will have been transformed by the
    registered transform function (see [<canvas_register_transform_func>]).
    [RETURNS] TRUE if the event was consumed, else FALSE indicating that
    the event is still to be processed.
*/
{
    CONST char *string = e_info;
    extern flag verbose;

    if (event_code != K_CANVAS_EVENT_PLAIN_KEY_PRESS) return (FALSE);
    if (strcmp (string, "v") != 0) return (FALSE);
    verbose = verbose ? FALSE : TRUE;
    fprintf (stderr, "Verbose (debug) mode %s\n", verbose ? "ON" : "OFF");
    return (TRUE);
}   /*  End Function position_func  */

#ifdef OS_SunOS

static void dummy ()
/*  [PURPOSE] This routine is needed for SunOS, because of a problem with
    shared archive use. The <<_XtToolkitInitialize>> symbol is defined in
    the Xt shared archive, but it appears the shared archive is not linked for
    this application. Other applications (like <<krenzo>>) have no such
    problems. This is a truly bizzare bug.
*/
{
    XtToolkitInitialize ();
}   /*  End Function dummy  */
#endif
