/*  scroll.c

    Scroll file for  MultibeamView  (X11 image/movie display tool for Karma).

    Copyright (C) 1997-2004  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*
    This Karma module will enable on-screen imaging.
    This module runs on an X11 server.


    Written by      Richard Gooch   18-APR-1997: Copied from glish.cc

    Updated by      Richard Gooch   23-APR-1997: Added <<scale>> paramater to
  <copy_line_of_doubles> function.

    Last updated by Richard Gooch   14-APR-2004: Removed scroll functions as
  <iarray_scroll> performs this function.


*/
#include <stdio.h>
#include <karma.h>
#include <karma_iarray.h>
#include <karma_wcs.h>
#include <karma_ds.h>
#include <karma_st.h>
#include <karma_ex.h>
#include <karma_m.h>
#include <karma_a.h>
#include "kview.h"
#include "MultibeamView.h"


/*  Public functions follow  */

void copy_plane (iarray array, unsigned long plane_index,
		 CONST void *in, double blank)
/*  [SUMMARY] Copy a plane of data to an Intelligent Array.
    <array> The Intelligent Array to write to.
    <plain_index> The index of the plane to write.
    <in> A pointer to the input data.
    <blank> The blank value in the input.
    [RETURNS] Nothing.
*/
{
    signed int blank_i, val_i;
    unsigned long xlen, ylen, x, y;
    float blank_f, val_f;
    double val_d;
    CONST signed int *ptr_i;
    CONST float *ptr_f;
    CONST double *ptr_d;
    static char function_name[] = "copy_plane";

    xlen = iarray_dim_length (array, 2);
    ylen = iarray_dim_length (array, 1);
    switch ( iarray_type (array) )
    {
      case K_INT:
	blank_i = (signed int) blank;
	ptr_i = (CONST signed int *) in;
	for (y = 0; y < ylen; ++y) for (x = 0; x < xlen; ++x, ++ptr_i)
	{
	    if ( (val_i = *ptr_i) == blank_i )
	    {
		I3 (array, plane_index, y, x) = -2147483647 - 1;
	    }
	    else I3 (array, plane_index, y, x) = val_i;
	}
	break;
      case K_FLOAT:
	blank_f = blank;
	ptr_f = (CONST float *) in;
	for (y = 0; y < ylen; ++y) for (x = 0; x < xlen; ++x, ++ptr_f)
	{
	    if ( (val_f = *ptr_f) == blank_f )
	    {
		F3 (array, plane_index, y, x) = TOOBIG;
	    }
	    else F3 (array, plane_index, y, x) = val_f;
	}
	break;
      case K_DOUBLE:
	ptr_d = (CONST double *) in;
	for (y = 0; y < ylen; ++y) for (x = 0; x < xlen; ++x, ++ptr_d)
	{
	    if ( (val_d = *ptr_d) == blank )
	    {
		D3 (array, plane_index, y, x) = TOOBIG;
	    }
	    else D3 (array, plane_index, y, x) = val_d;
	}
	break;
      default:
	fprintf ( stderr, "Illegal data type: %u\n", iarray_type (array) );
	a_prog_bug (function_name);
	break;
    }
}   /*  End Function copy_plane  */

void copy_line_of_doubles (iarray array, CONST double *data, uaddr ypos,
			   double scale)
/*  [SUMMARY] Copy a line of doubles to a 2D array.
    <array> The 2D array to write to.
    <data> The line of data to read.
    <ypos> The position of the line in the array.
    <scale> The scale value to multiply the input data by.
    [RETURNS] Nothing.
*/
{
    uaddr xpos, xlen;

    xlen = iarray_dim_length (array, 1);
    for (xpos = 0; xpos < xlen; ++xpos)
	D2 (array, ypos, xpos) = data[xpos] * scale;
}   /*  End Function copy_line_of_doubles  */
