/*  kpvslice.h

    Include file for  kpvslice  (X11 Position-Velocity slice tool for Karma).

    Copyright (C) 1996-2002  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*
    This Karma module will enable interactive selection of a Position-Velocity
    slice through a data cube.
    This module runs on an X11 server.


    Written by      Richard Gooch   13-AUG-1996

    Last updated by Richard Gooch   5-JAN-2002: Moved pvget declarations out.


*/
#include <karma.h>
#include <karma_viewimg_def.h>
#include <karma_canvas_def.h>
#include <karma_iarray_def.h>
#include <karma_kcmap_def.h>
#include <karma_kwin_def.h>
#include <karma_psw_def.h>
#include <karma_wcs.h>
#include <k_win_scale.h>

#ifndef WIDGET_TYPE
#  define WIDGET_TYPE void *
#endif


#define IMAGE_MODE_LOADED             0
#define IMAGE_MODE_MOM0               1
#define IMAGE_MODE_MOM1               2
#define IMAGE_MODE_CHANNEL0           3
#define NUM_IMAGE_MODE_ALTERNATIVES   4

#ifdef NEED_IMAGE_MODE_ALTERNATIVES
static char *image_mode_alternatives[] =
{
    "loaded image",
    "0th moment",
    "1st moment",
    "0th channel",
};
#endif

typedef struct slice_display
{
    iarray cube;
    ViewableImage vimage;
    iarray image;
    KWorldCanvas canvas;
    WIDGET_TYPE shell;                 /*  Xt widget                         */
    WIDGET_TYPE image_display;
    WIDGET_TYPE ztrace_winpopup;
    flag show_trace;
    flag had_cube;                     /*  TRUE if ever a cube was loaded    */
    KCallbackFunc cube_callback;
    KCallbackFunc main_canvas_event_callback;
    double cube_min;
    double cube_max;
    unsigned int num_points_in_slice;  /*  Number of valid values per array  */
    /*  Arrays of data  */
    double *offset_coords;             /*  Units: Offset degrees             */
    double *x_locus;                   /*  Units: cube co-ord indices        */
    double *y_locus;                   /*  Units: cube co-ord indices        */
    double *ra_coord;                  /*  Units: degrees                    */
    double *dec_coord;                 /*  Units: degrees                    */
    struct slice_display *prev;
    struct slice_display *next;
    char domain_name[32];
} *SliceDisplay;

/*  File: main_xt.c  */
EXTERN_FUNCTION (void print_slice_info, (CONST char *window_string) );
EXTERN_FUNCTION (void update_trace_profile,
		 (double x0, double y0, double x1, double y1) );
EXTERN_FUNCTION (void new_primary_cube,
		 (iarray cube, double min, double max) );

/*  File: main_xt.c  */
EXTERN_FUNCTION (flag track_aux_canvas_xtcoord,
		 (SliceDisplay sd, CONST char *track_txt, int track_num) );

/*  File: generic.c  */
EXTERN_FUNCTION (flag process_image,
		 (multi_array *multi_desc, iarray *image_arr,
		  double *min, double *max,
		  KWorldCanvas pseudo_canvas, KWorldCanvas mag_pseudo_canvas,
		  ViewableImage *image, ViewableImage *magnified_image) );
EXTERN_FUNCTION (flag process_cube,
		 (multi_array *multi_desc, iarray *cube_arr,
		  double *min, double *max) );
EXTERN_FUNCTION (flag copy_header_info, (iarray out, iarray in) );
EXTERN_FUNCTION (void convert_lin_world_coords,
		 (double *xout, double *yout, KwcsAstro out_ap,
		  CONST double *xin, CONST double *yin, KwcsAstro in_ap,
		  unsigned int num_coords) );

/*  File: event.c  */
EXTERN_FUNCTION (flag endpoint_position_func,
		 (KWorldCanvas canvas, double x, double y,
		  unsigned int event_code, void *e_info,
		  void **f_info, double x_lin, double y_lin) );
EXTERN_FUNCTION (flag track_aux_canvas_event,
		 (ViewableImage vimage, double x, double y,
		  void *value, unsigned int event_code,
		  void *e_info, void **f_info,
		  double x_lin, double y_lin, unsigned int value_type,
		  double x_im, double y_im, int x_pix, int y_pix) );
EXTERN_FUNCTION (void main_refresh_func,
		 (KWorldCanvas canvas, int width, int height,
		  struct win_scale_type *win_scale,
		  Kcolourmap cmap, flag cmap_resize, void **info,
		  PostScriptPage pspage,
		  unsigned int num_areas, KPixCanvasRefreshArea *areas,
		  flag *honoured_areas) );
EXTERN_FUNCTION (void trace_refresh_func,
		 (KWorldCanvas canvas, int width, int height,
		  struct win_scale_type *win_scale,
		  Kcolourmap cmap, flag cmap_resize, void **info,
		  PostScriptPage pspage,
		  unsigned int num_areas, KPixCanvasRefreshArea *areas,
		  flag *honoured_areas) );
