/*  main_xt.c

    Main file for  kscale  (X11 data scaling tool for Karma).

    Copyright (C) 1997-2003  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*
    This Karma module will load and scale data.
    This module runs on an X11 server.


    Written by      Richard Gooch   6-NOV-1997: Copied from kshell/main_xt.c

    Updated by      Richard Gooch   6-NOV-1997

    Updated by      Richard Gooch   1-MAR-1998: Made use of new filewinWidget
  resource.

    Updated by      Richard Gooch   11-MAR-1998: Enable blank toggle.

    Updated by      Richard Gooch   30-AUG-1999: Added dirname font resource.

    Updated by      Richard Gooch   11-MAY-2001: Changed dirname font resource.

    Updated by      Richard Gooch   3-OCT-2001: Added translations for Filewin.

    Updated by      Richard Gooch   21-DEC-2001: Included k_version.h directly.

    Updated by      Richard Gooch   22-DEC-2002: Added #include <stdlib.h>

    Last updated by Richard Gooch   22-DEC-2002: Added
  #include <karma_iarray.h>


*/
#include <stdlib.h>
#include <stdio.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <karma.h>
#include <k_version.h>
#include <karma_xtmisc.h>
#include <karma_iarray.h>
#include <karma_ds.h>
#include <karma_im.h>
#include <karma_a.h>
#include <Xkw/Dataclip.h>
#include <Xkw/Filepopup.h>
#include <Xkw/Filewin.h>


#define VERSION "1.0.4"


/*  Private functions  */
STATIC_FUNCTION (void quit_cbk, (Widget w, XtPointer client_data,
				 XtPointer call_data) );
STATIC_FUNCTION (flag data_event_func,
		 (void *object, multi_array *multi_desc,
		  CONST char *domain, CONST char *name) );


/*  Private data  */
String fallback_resources[] =
{
    "Kscale*pseudoColourCanvas*background:            black",
    "Kscale*directColourCanvas*background:            black",
    "Kscale*trueColourCanvas*background:              black",
    "Kscale*pseudoColourCanvas*foreground:            white",
    "Kscale*Command*background:                       grey70",
    "Kscale*Repeater*background:                      grey70",
    "Kscale*Ktoggle*background:                       grey80",
    "Kscale*closeButton*background:                   grey90",
    "Kscale*ChoiceMenu.background:                    turquoise",
    "Kscale*ExclusiveMenu.background:                 turquoise",
    "Kscale*Value*background:                         #d0a0a0",
    "Kscale*topForm*quit*background:                  orange",
    "Kscale*ScrollableList.highlightPixel:            Sea Green",
    "Kscale*SimpleSlider.foreground:                  Sea Green",
    "Kscale*ExclusiveMenu.font:                       8x13bold",
    "Kscale*Dataclip.AutoValueScale:                  False",
    "Kscale*Dataclip.showBlankControl:                True",
    "Kscale*SimpleSlider.borderWidth:                 0",
    "Kscale*Filewin.dirname.textSink.font:            6x13",
    "Kscale*font:                                     9x15bold",
    "Kscale*borderColor:                              black",
    "Kscale*background:                               aquamarine",
    "Kscale*foreground:                               black",
    NULL
};
static Widget izoomwinpopup = NULL;
/*static char title_name[STRING_LENGTH];*/

/*  Public functions follow  */

int main (int argc, char **argv)
{
    XtAppContext app_context;
    Widget main_shell, topform, filewin, filepopup, w, files_btn;
    /*extern char title_name[STRING_LENGTH];*/

    /*  Initialise module  */
    im_register_module_name ("kscale");
    im_register_module_version_date (VERSION);
    im_register_lib_version (KARMA_VERSION);
    /*  Start up Xt  */
    main_shell = xtmisc_init_app_initialise (&app_context, "Kscale",
					     NULL, 0,
					     &argc, argv, fallback_resources,
					     XTMISC_INIT_ATT_COMMS_SETUP, TRUE,
					     XTMISC_INIT_ATT_END,
					     NULL);
    /*xtmisc_set_icon (main_shell, ic_write_kimage_icon);*/
    /*XtVaSetValues (main_shell,
		   XtNtitle, title_name,
		   NULL);*/
    topform = XtVaCreateManagedWidget ("topForm", formWidgetClass, main_shell,
				       XtNborderWidth, 0,
				       NULL);
    /*  Create the file selector and popup button  */
    filepopup = XtVaCreatePopupShell ("filewinPopup", filepopupWidgetClass,
				      main_shell,
				      XtNtitle, "Kscale Image Selector",
				      XtNx, 0,
				      XtNy, 0,
				      XkwNautoPopdown, True,
				      XkwNfilenameTester,
				      XkwFilewinStandardFileTester_nD,
				      XkwNdomainName, "IMAGE",
				      XkwNloadFiles, TRUE,
				      NULL);
    files_btn = XtVaCreateManagedWidget ("filesButton", commandWidgetClass,
					 topform,
					 XtNlabel, "Files",
					 NULL);
    XtAddCallback (files_btn, XtNcallback, xtmisc_popup_cbk, filepopup);
    XtVaGetValues (filepopup, XkwNfilewinWidget, &filewin, NULL);
    XtVaSetValues (filewin,
		   XkwNtrapDirectoryDatasets, True,
		   NULL);
    ds_event_register_func (data_event_func, NULL);
    w = XtVaCreateManagedWidget ("quit", commandWidgetClass, topform,
				 XtNlabel, "Quit",
				 XtNfromHoriz, files_btn,
				 NULL);
    XtAddCallback (w, XtNcallback, quit_cbk, NULL);
    XtRealizeWidget (main_shell);
    izoomwinpopup = XtVaCreatePopupShell ("izoomwinpopup", dataclipWidgetClass,
					  main_shell,
					  XtNtitle, "Scale Control",
					  NULL);
    XtAppMainLoop (app_context);
    return (RV_OK);
}   /*  End Function main   */


/*  Private routines follow  */

static void quit_cbk (Widget w, XtPointer client_data, XtPointer call_data)
/*  This is the quit button callback.
*/
{
    exit (RV_OK);
}   /*  End Function quit_cbk   */

static flag data_event_func (void *object, multi_array *multi_desc,
			     CONST char *domain, CONST char *name)
/*  [SUMMARY] Process a new data event.
    <object> The object information pointer.
    <multi_desc> The multi_array descriptor.
    <domain> The domain from where the event was generated.
    <name> The name of the event in its domain.
    [RETURNS] TRUE if further callbacks should not be called, else FALSE.
*/
{
    static iarray array = NULL;
    static char function_name[] = "data_event_func";

    XtPopup (izoomwinpopup, XtGrabNone);
    if ( !iarray_create_and_setup (&array, multi_desc, FALSE, 0, NONE,
				   NULL, NULL, FALSE, NULL) )
    {
	fprintf (stderr, "%s: error getting iarray\n", function_name);
	return (TRUE);
    }
    XkwDataclipNewArray (izoomwinpopup, array, TOOBIG, -TOOBIG, TRUE);
    return (TRUE);
}   /*  End Function data_event_func  */
