/*  display_xt.c

    Source file for  xray  (Karma volume rendering tool).

    Copyright (C) 1996-2004  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*
    This file contains the code necessary for creating the image display window


    Written by      Richard Gooch   23-AUG-1996: Copied from main_xt.c and made
  use of new cursor move event list.

    Updated by      Richard Gooch   8-DEC-1996: Minor cosmetic changes. Removed
  hack for Sung Kim where 2-D active cursor could be written: better to do this
  in the TracePopup widget.

    Updated by      Richard Gooch   10-DEC-1996: Switched to
  <register_cursor_func> now that wireframe for rendered images is done using
  overlay lists.

    Updated by      Richard Gooch   12-JAN-1997: Made use of new
  XkwNtrapDirectoryDatasets resource for FilewinWidgetClass.

    Updated by      Richard Gooch   22-APR-1997: Chaged to prefer TrueColour
  visual rather than DirectColour.

    Updated by      Richard Gooch   9-DEC-1997: Created cursor and region
  subvolume render popups.

    Updated by      Richard Gooch   1-MAR-1998: Made use of new filewinWidget
  resource.

    Updated by      Richard Gooch   17-JUN-1998: Created subvolume and region
  render popups.

    Updated by      Richard Gooch   28-AUG-1998: Moved region render popup
  creation to region_xt.c

    Updated by      Richard Gooch   31-AUG-1998: Moved cursor render popup
  creation to cursor_3d_xt.c

    Updated by      Richard Gooch   8-OCT-1998: Make stereo toggle insensitive
  if no stereo canvas available.

    Updated by      Richard Gooch   26-JAN-2000: Removed unnecessary includes.

    Updated by      Richard Gooch   22-DEC-2002: Stupid compiler warning fix.

    Updated by      Richard Gooch   21-MAR-2003: Switched to function
  <overlay_create_viewable>.

    Updated by      Richard Gooch   16-JUN-2003: #include cleanups.

    Last updated by Richard Gooch   29-MAR-2004: Took account of ImageDisplay
  resource changes.


*/
#include <stdio.h>
#include <errno.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <karma.h>
#ifndef X11
#  define X11
#endif
#include <k_event_codes.h>
#include <karma_overlay.h>
#include <karma_vrender.h>
#include <karma_viewimg.h>
#include <karma_canvas.h>
#include <karma_xtmisc.h>
#include <karma_ic.h>
#include <karma_a.h>
#include <karma_m.h>
#include <karma_c.h>
#include <Xkw/ImageDisplay.h>
#include <Xkw/Filewin.h>
#include "xraycast_3d.h"


/*  Private functions  */
STATIC_FUNCTION (flag track_canvas_event,
		 (ViewableImage vimage, double x, double y,
		  void *value, unsigned int event_code,
		  void *e_info, void **f_info,
		  double x_lin, double y_lin, unsigned int value_type) );
STATIC_FUNCTION (void new_frame_cbk, (Widget w, XtPointer client_data,
				      XtPointer call_data) );
STATIC_FUNCTION (void dispfileselect_cbk, (Widget w, XtPointer client_data,
					   XtPointer call_data) );
STATIC_FUNCTION (flag cursor_callback,
		 (void *object, void *client1_data, void *call_data,
		  void *client2_data) );


/*  Private data  */


/*  Public functions follow  */

Widget create_display_win (Widget main_shell, Widget stereo_tgl)
{
    KWorldCanvas direct_wc, left_direct_wc, right_direct_wc;
    KWorldCanvas true_wc, left_true_wc, right_true_wc;
    KViewableOverlayList vlist;
    Widget track_label, filepopup, filewin, animate_winpopup;
    Widget fullscreen;
    Widget popup;
    extern KWorldCanvas cyclops_image_pc_worldcanvas;
    extern KWorldCanvas cyclops_image_rgb_worldcanvas;
    extern KWorldCanvas left_image_pc_worldcanvas, right_image_pc_worldcanvas;
    extern KWorldCanvas left_image_rgb_worldcanvas,right_image_rgb_worldcanvas;
    extern KOverlayList cyclops_olist, left_olist, right_olist;
    extern KCallbackList cursor3d_event_list;
    extern KVolumeRenderContext main_vrend_ctx;
    extern flag verbose;
    extern Widget image_display;

    popup = XtVaCreatePopupShell ("displaywinpopup",
				  topLevelShellWidgetClass, main_shell,
				  XtNtitle, "Display Window",
				  NULL);
    xtmisc_set_icon (popup, ic_write_kimage_icon);
    image_display = XtVaCreateManagedWidget ("form", imageDisplayWidgetClass,
					     popup,
					     XtNborderWidth, 0,
					     XkwNenableAnimation, True,
					     XkwNshowQuitButton, False,
					     XkwNnumTrackLabels, 3,
					     NULL);
    XtRealizeWidget (popup);
    fullscreen = XtNameToWidget (image_display, "fullscreenPopup");
    if (fullscreen != NULL)
    {
	Screen *screen;

	screen = XtScreen (fullscreen);
	XLowerWindow ( XtDisplay (fullscreen), XtWindow (fullscreen) );
	XLowerWindow ( XtDisplay (fullscreen), RootWindowOfScreen (screen) );
    }
    XtVaGetValues (image_display,
		   XkwNpseudoColourCanvas, &cyclops_image_pc_worldcanvas,
		   XkwNpseudoColourLeftCanvas, &left_image_pc_worldcanvas,
		   XkwNpseudoColourRightCanvas, &right_image_pc_worldcanvas,
		   XkwNdirectColourCanvas, &direct_wc,
		   XkwNdirectColourLeftCanvas, &left_direct_wc,
		   XkwNdirectColourRightCanvas, &right_direct_wc,
		   XkwNtrueColourCanvas, &true_wc,
		   XkwNtrueColourLeftCanvas, &left_true_wc,
		   XkwNtrueColourRightCanvas, &right_true_wc,
		   NULL);
    filepopup = XtNameToWidget (image_display, "filewinPopup");
    XtVaGetValues (filepopup, XkwNfilewinWidget, &filewin, NULL);
    XtVaSetValues (filepopup,
		   XkwNfilenameTester, XkwFilewinStandardFileTester_nD,
		   NULL);
    XtAddCallback (filepopup, XkwNfileSelectCallback, dispfileselect_cbk,NULL);
    XtVaSetValues (filewin,
		   XkwNtrapDirectoryDatasets, True,
		   NULL);
    animate_winpopup = XtNameToWidget (image_display, "animatepopup");
    XtAddCallback (animate_winpopup, XkwNnewFrameCallback, new_frame_cbk,NULL);
    /*  Prefer TrueColour visual  */
    cyclops_image_rgb_worldcanvas = (true_wc == NULL) ? direct_wc : true_wc;
    if (left_true_wc == NULL)
    {
	left_image_rgb_worldcanvas = left_direct_wc;
	right_image_rgb_worldcanvas = right_direct_wc;
    }
    else
    {
	left_image_rgb_worldcanvas = left_true_wc;
	right_image_rgb_worldcanvas = right_true_wc;
    }
    if (verbose)
    {
	fprintf (stderr, "leftRGB: %p  rightRGB: %p\n",
		 (void *) left_image_rgb_worldcanvas,
		 (void *) right_image_rgb_worldcanvas);
    }
    if ( (left_image_pc_worldcanvas == NULL) &&
	 (left_image_rgb_worldcanvas == NULL) )
	XtSetSensitive (stereo_tgl, False);
    track_label = XtNameToWidget (image_display, "trackLabel0");
    if (cyclops_image_pc_worldcanvas)
    {
	viewimg_register_position_event_func (cyclops_image_pc_worldcanvas,
					      track_canvas_event,
					      (void *) track_label);
	canvas_register_position_event_func (cyclops_image_pc_worldcanvas,
					     cursor_3d_position_func,
					     (void *) VRENDER_EYE_CHOICE_CYCLOPS);
	vlist = overlay_create_viewable (cyclops_olist,
					 cyclops_image_pc_worldcanvas);
	overlay_set_active (vlist, TRUE, FALSE, TRUE, FALSE, 0);
    }
    if (left_image_pc_worldcanvas)
    {
	viewimg_register_position_event_func (left_image_pc_worldcanvas,
					      track_canvas_event,
					      (void *) track_label);
	canvas_register_position_event_func (left_image_pc_worldcanvas,
					     cursor_3d_position_func,
					     (void *) VRENDER_EYE_CHOICE_LEFT);
	vlist = overlay_create_viewable (left_olist,
					 left_image_pc_worldcanvas);
	overlay_set_active (vlist, TRUE, FALSE, TRUE, FALSE, 0);
	viewimg_register_position_event_func (right_image_pc_worldcanvas,
					      track_canvas_event,
					      (void *) track_label);
	vlist = overlay_create_viewable (right_olist,
					 right_image_pc_worldcanvas);
	overlay_set_active (vlist, TRUE, FALSE, TRUE, FALSE, 0);
    }
    if (cyclops_image_rgb_worldcanvas)
    {
	viewimg_register_position_event_func (cyclops_image_rgb_worldcanvas,
					      track_canvas_event,
					      (void *) track_label);
	canvas_register_position_event_func (cyclops_image_rgb_worldcanvas,
					     cursor_3d_position_func,
					     (void *) VRENDER_EYE_CHOICE_CYCLOPS);
	vlist = overlay_create_viewable (cyclops_olist,
					 cyclops_image_rgb_worldcanvas);
	overlay_set_active (vlist, TRUE, FALSE, TRUE, FALSE, 0);
    }
    if (left_image_rgb_worldcanvas)
    {
	viewimg_register_position_event_func (left_image_rgb_worldcanvas,
					      track_canvas_event,
					      (void *) track_label);
	canvas_register_position_event_func (left_image_rgb_worldcanvas,
					     cursor_3d_position_func,
					     (void *) VRENDER_EYE_CHOICE_LEFT);
	vlist = overlay_create_viewable (left_olist,
					 left_image_rgb_worldcanvas);
	overlay_set_active (vlist, TRUE, FALSE, TRUE, FALSE, 0);
	viewimg_register_position_event_func (right_image_rgb_worldcanvas,
					      track_canvas_event,
					      (void *) track_label);
	vlist = overlay_create_viewable (right_olist,
					 right_image_rgb_worldcanvas);
	overlay_set_active (vlist, TRUE, FALSE, TRUE, FALSE, 0);
    }
    /*  Register refresh functions for the wireframe on all canvases  */
    register_cursor_func (cyclops_image_pc_worldcanvas, main_vrend_ctx,
			  VRENDER_EYE_CHOICE_CYCLOPS, TRUE);
    register_cursor_func (left_image_pc_worldcanvas, main_vrend_ctx,
			  VRENDER_EYE_CHOICE_LEFT, TRUE);
    register_cursor_func (right_image_pc_worldcanvas, main_vrend_ctx,
			  VRENDER_EYE_CHOICE_RIGHT, TRUE);
    register_cursor_func (direct_wc, main_vrend_ctx,
			  VRENDER_EYE_CHOICE_CYCLOPS, TRUE);
    register_cursor_func (left_direct_wc, main_vrend_ctx,
			  VRENDER_EYE_CHOICE_LEFT, TRUE);
    register_cursor_func (right_direct_wc, main_vrend_ctx,
			  VRENDER_EYE_CHOICE_RIGHT, TRUE);
    register_cursor_func (true_wc, main_vrend_ctx,
			  VRENDER_EYE_CHOICE_CYCLOPS, TRUE);
    register_cursor_func (left_true_wc, main_vrend_ctx,
			  VRENDER_EYE_CHOICE_LEFT, TRUE);
    register_cursor_func (right_true_wc, main_vrend_ctx,
			  VRENDER_EYE_CHOICE_RIGHT, TRUE);
    c_register_callback (&cursor3d_event_list, cursor_callback, image_display,
			 NULL, FALSE, NULL, FALSE, FALSE);
    return (popup);
}   /*  End Function create_display_win  */


/*  Private functions follow  */

static flag track_canvas_event (ViewableImage vimage, double x, double y,
				void *value, unsigned int event_code,
				void *e_info, void **f_info,
				double x_lin, double y_lin,
				unsigned int value_type)
/*  [PURPOSE] This routine is a position event consumer for a world canvas
    which has a number of ViewableImage objects associated with it.
    <viewimg> The active viewable image.
    <x> The horizontal world co-ordinate of the event.
    <y> The vertical world co-ordinate of the event.
    These values will have been transformed by the registered transform
    function (see  canvas_register_transform_func  ) for the associated
    world canvas.
    <value> A pointer to the data value in the viewable image corresponding
    to the event co-ordinates.
    <event_code> The arbitrary event code.
    <e_info> The arbitrary event information.
    <f_info> The arbitrary function information pointer.
    <x_lin> The linear horizontal world co-ordinate (the co-ordinate prior
    to the transform function being called).
    <y_lin> The linear vertical world co-ordinate (the co-ordinate prior
    to the transform function being called).
    <value_type> The type of the data value. This may be K_DCOMPLEX or
    K_UB_RGB.
    [RETURNS] TRUE if the event was consumed, else FALSE indicating that
    the event is still to be processed.
*/
{
    unsigned int ray_length, depth;
    Widget track_label = (Widget) *f_info;
    Kcoord_3d coords;
    Kcoord_2d pos;
    CONST signed char *ray;
    unsigned char *rgb_ptr = (unsigned char *) value;
    char txt[STRING_LENGTH];
    char window_string[STRING_LENGTH];
    float opacities[4096];
    extern KOverlayList ray_display_olist;
    extern flag stereo;
    extern unsigned int cursor_2d_mode;
    extern CONST char *current_shader_name;
    static char function_name[] = "track_canvas_event";

    switch (event_code)
    {
      case K_CANVAS_EVENT_POINTER_MOVE:
	break;
      default:
	return (FALSE);
	/*break;*/
    }
    pos.x = x;
    pos.y = y;
    switch (cursor_2d_mode)
    {
      case CURSOR_2D_DUMB:
	switch (value_type)
	{
	  case K_DCOMPLEX:
	    sprintf (window_string, "abs: %5e  ord: %5e      value: %e",
		     x, y, *(double *) value);
	    break;
	  case K_UB_RGB:
	    sprintf (window_string, "abs: %5e  ord: %5e      RGB: %u %u %u",
		     x, y, rgb_ptr[0], rgb_ptr[1], rgb_ptr[2]);
	    break;
	  default:
	    fprintf (stderr, "Illegal type: %u\n", value_type);
	    a_prog_bug (function_name);
	    break;
	}
	break;
      case CURSOR_2D_ACTIVE:
	if (stereo) break;
	overlay_remove_objects (ray_display_olist, 0);
	if ( ( ray = get_ray_from_image_plane (&coords, pos, &ray_length,
					       current_shader_name, &depth,
					       opacities) )
	    == NULL )
	{
	    sprintf (window_string, "No intersection");
	}
	else
	{
	    compute_cursor_3d_strings (coords, NULL, txt);
	    sprintf (window_string, "2D-A %s", txt);
	    register_ray (ray, ray_length, depth, opacities);
	}
	break;
      default:
	fprintf (stderr, "Illegal cursor mode: %u\n", cursor_2d_mode);
	a_prog_bug (function_name);
	break;
    }
    XtVaSetValues (track_label, XtNlabel, window_string, NULL);
    m_clear (window_string, STRING_LENGTH);
    return (TRUE);
}   /*  End Function track_canvas_event  */

static void new_frame_cbk (Widget w, XtPointer client_data,XtPointer call_data)
/*  This is the new frame callback.
*/
{
    int frame_number = *(int *) call_data;
    extern unsigned int num_frames;
    extern ViewableImage *vimages_left, *vimages_right;

    if (frame_number >= num_frames)
    {
	fprintf (stderr, "Frame number: %d larger than number of frames: %d\n",
		 frame_number, num_frames);
	return;
    }
    if (vimages_left[frame_number] == NULL)
    {
	fprintf (stderr, "Frame: %d does not exist!\n", frame_number);
	return;
    }
    viewimg_make_active (vimages_left[frame_number]);
    if (vimages_right[frame_number] != NULL)
    {
	viewimg_make_active (vimages_right[frame_number]);
    }
}   /*  End Function new_frame_cbk   */

static void dispfileselect_cbk (Widget w, XtPointer client_data,
				XtPointer call_data)
{
    Widget animate_winpopup;
    char *filename, *ptr;
    char stripped_filename[STRING_LENGTH];
    extern KWorldCanvas cyclops_image_pc_worldcanvas;
    extern KWorldCanvas cyclops_image_rgb_worldcanvas;
    extern unsigned int num_frames;
    extern Widget image_display;

    filename = (char *) call_data;
    if ( !display_file (filename, cyclops_image_pc_worldcanvas,
			cyclops_image_rgb_worldcanvas) )
    {
	return;
    }
    strcpy (stripped_filename, filename);
    if ( ( ptr = strrchr (stripped_filename, '.') ) != NULL )
    {
	if (strcmp (ptr, ".kf") == 0) *ptr = '\0';
    }
    XtVaSetValues (image_display,
		   XkwNimageName, stripped_filename,
		   NULL);
    animate_winpopup = XtNameToWidget (image_display, "animatepopup");
    XtVaSetValues (animate_winpopup, XkwNnumFrames, num_frames, NULL);
    if (num_frames > 1) show_animate_control (TRUE);
    else show_animate_control (FALSE);
}   /*  End Function dispfileselect_cbk  */

static flag cursor_callback (void *object, void *client1_data, void *call_data,
			     void *client2_data)
/*  [SUMMARY] Generic callback.
    [PURPOSE] This routine is called when object callbacks are called.
    <object> The object information pointer.
    <client1_data> The first client information pointer.
    <call_data> The call information pointer.
    <client2_data> The second client information pointer.
    [RETURNS] TRUE if further callbacks should not be called, else FALSE.
*/
{
    Widget image_display = (Widget) object;
    Widget track_label;
    Kcoord_3d *cursor = (Kcoord_3d *) call_data;
    char pixel_string[STRING_LENGTH], world_string[STRING_LENGTH + 1];
    char txt[STRING_LENGTH];

    compute_cursor_3d_strings (*cursor, pixel_string, world_string);
    sprintf (txt, "3D %s", pixel_string);
    track_label = XtNameToWidget (image_display, "trackLabel1");
    XtVaSetValues (track_label, XtNlabel, txt, NULL);
    sprintf (txt, "3D %s", world_string);
    track_label = XtNameToWidget (image_display, "trackLabel2");
    XtVaSetValues (track_label, XtNlabel, txt, NULL);
    return (FALSE);
}   /*  End Function cursor_callback  */
