#define   INTERPOL  

#include <karma_iarray.h>
#include <karma_m.h>

#include "memutil.h"
#include "interpol.h"

static iarray tmp_array = NULL;

flag iarray_expand_float (iarray in_array, iarray out_array, int exp_fac,
			  int edge_method)
{
    /*  
       Expands an 2d iarray with an integral factor, using anti aliasing
       interpolation. Returns TRUE on succes, else FALSE. 
       
       iarray    in_array      input image
       iarray    out_array     output image
       int       exp_fac       expand factor for image
       int       edge_method   defines how edges are handled
       
       Updates:    13-Nov-94   Created from old code
       15-Nov-94   Added handling of edges
       */
              
    int     j, k, xlen, ylen, out_xlen, out_ylen;
    int     ii, xco, yco, up_x, up_y;
    float   *y, **w;

    if (exp_fac < 2) {
	return(FALSE);
    }
  
    if (in_array == (iarray) NULL) { /* checks on input   */
	return(FALSE);
    }
  
    if (out_array == (iarray) NULL) {
	return(FALSE);
    }
 
    if (iarray_num_dim(in_array) != 2) {
	return(FALSE);
    }

    if (iarray_num_dim(out_array) != 2) {
	return(FALSE);
    }

    if (iarray_type(in_array) != K_FLOAT) {
	return(FALSE);
    }

    if (iarray_type(out_array) != K_FLOAT) {
	return(FALSE);
    }
    
    xlen = iarray_dim_length(in_array, 1); 
    ylen = iarray_dim_length(in_array, 0);
    out_xlen = iarray_dim_length(out_array, 1); 
    out_ylen = iarray_dim_length(out_array, 0);

    if (xlen*exp_fac > out_xlen) {
	return(FALSE);
    }
    if (ylen*exp_fac > out_ylen) {
	return(FALSE);
    }

    y = mal_vector(-1, 2);	/* create work arrays */
    w = mal_matrix(1, exp_fac-1, -1, 2);
  
    setup_interpol(w, exp_fac);	/* setup interpolation coefficients */
  
    up_x = xlen - 2;
    up_y = ylen;

    for (j = 0; j < up_y; j++) { /* first loop: interpolate in X */
	yco = exp_fac*j;

	y[0]  = F2(in_array, j, 0); /* handle left edge */
	y[1]  = F2(in_array, j, 1);
	y[2]  = F2(in_array, j, 2);
	switch (edge_method) {	/* how to treat edges */
	  case INTER_EDGE_SAME : {
	      y[-1] = y[0];	/* same as edge data */
	      break;
	  } 
	  case INTER_EDGE_MIRROR : { /* mirror image in X */
	      y[-1] = F2(in_array, j, xlen-1);
	      break;
	  }
	  case INTER_EDGE_ZERO : /* set to zero */
	  default : {
	      y[-1] = 0.0;
	      break;
	  }
	}
 
	F2(out_array, yco, 0) = y[0]; /* do not interpolate where you don't need */
	for (ii = 1; ii < exp_fac; ii++) { /* interpolate */
	    F2(out_array, yco, ii) = interpol(y, w, ii);
	} /* and store in outout image */
    
	for (k = 1; k < up_x; k++) { /* central part */
	    y[-1] = F2(in_array, j, k-1);
	    y[0]  = F2(in_array, j, k);
	    y[1]  = F2(in_array, j, k+1);
	    y[2]  = F2(in_array, j, k+2);
      
	    xco = exp_fac*k;

	    F2(out_array, yco, xco) = y[0];
	    for (ii = 1; ii < exp_fac; ii++) {
		F2(out_array, yco, xco+ii) = interpol(y, w, ii);
	    }
	}

	y[-1] = F2(in_array, j, xlen - 3); /* first right  edge of image */
	y[0]  = F2(in_array, j, xlen - 2);
	y[1]  = F2(in_array, j, xlen - 1);

	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[2] = F2(in_array, j, 0);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = 0.0;
	      break;
	  }
	}
      
	xco = exp_fac*(xlen-2);

	F2(out_array, yco, xco) = y[0];
	for (ii = 1; ii < exp_fac; ii++) {
	    F2(out_array, yco, xco+ii) = interpol(y, w, ii);
	}
    

	y[-1] = F2(in_array, j, xlen - 2); /* second right  edge of image */
	y[0]  = F2(in_array, j, xlen - 1);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1] = y[0];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[1] = F2(in_array, j, 0);
	      y[2] = F2(in_array, j, 1);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = y[1] = 0.0;
	      break;
	  }
	}

	xco = exp_fac*(xlen-1);

	F2(out_array, yco, xco) = y[0];
	for (ii = 1; ii < exp_fac; ii++) {
	    F2(out_array, yco, xco+ii) = interpol(y, w, ii);
	}
    
    }

    up_y = exp_fac*(ylen-2);
    up_x = exp_fac*xlen;
    /* second loop: interpolate in Y */
    for (k = 0; k < up_x; k++) { /* use data stored output image  */
	y[0]  = F2(out_array, 0, k); /* bottom edge */
	y[1]  = F2(out_array, exp_fac, k);
	y[2]  = F2(out_array, 2*exp_fac, k);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[-1] = y[0];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[-1] = F2(out_array, out_ylen-1, k);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[-1] = 0.0;
	      break;
	  }
	}


	for (ii = 1; ii < exp_fac; ii++) {
	    F2(out_array, ii, k) = interpol(y, w, ii);
	}
    }

    for (j = exp_fac; j < up_y; j  += exp_fac) { /* main part of image */
	for (k = 0; k < up_x; k++) {
	    y[-1] = F2(out_array, j-exp_fac, k);
	    y[0]  = F2(out_array, j, k);
	    y[1]  = F2(out_array, j+exp_fac, k);
	    y[2]  = F2(out_array, j+2*exp_fac, k);

	    for (ii = 1; ii < exp_fac; ii++) {
		F2(out_array, j+ii, k) = interpol(y, w, ii);
	    }
	}
    }

    yco = up_y;
  
    for (k = 0; k < up_x; k++) { /* first top row */
	y[-1] = F2(out_array, yco - exp_fac, k);
	y[0]  = F2(out_array, yco          , k);
	y[1]  = F2(out_array, yco + exp_fac, k);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[2] = F2(out_array, 0, k);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = 0.0;
	      break;
	  }
	}

	for (ii = 1; ii < exp_fac; ii++) {
	    F2(out_array, yco+ii, k) = interpol(y, w, ii);
	}
    }

    yco = yco + exp_fac;
    for (k = 0; k < up_x; k++) { /* second top row */
	y[-1] = F2(out_array, yco - exp_fac, k);
	y[0]  = F2(out_array, yco, k);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1] = y[0];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[1] = F2(out_array, 0, k);
	      y[2] = F2(out_array, exp_fac, k);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = y[1] = 0.0;
	      break;
	  }
	}

	for (ii = 1; ii < exp_fac; ii++) {
	    F2(out_array, yco+ii, k) = interpol(y, w, ii);
	}
    }

    free_vector(y, -1, 2);	/* clean up */
    free_matrix(w, 1, exp_fac-1, -1, 2);
  
    return(TRUE);
}

flag iarray_expand_byte (iarray in_array, iarray out_array, int exp_fac,
			 int edge_method)
/*  
   Expands an 2d iarray with an integral factor, using anti aliasing
   interpolation. Returns TRUE on succes, else FALSE. 

   iarray    in_array      input image
   iarray    out_array     output image
   int       exp_fac       expand factor for image
   int       edge_method   defines how edges are handled

   Updates:    13-Nov-94   Created from old code
   15-Nov-94   Added handling of edges
*/
{
              
    int     j, k, xlen, ylen, out_xlen, out_ylen;
    int     ii, xco, yco, up_x, up_y;
    float   *y, **w;
    static char function_name[] = "iarray_expand_byte";

    if (exp_fac < 2) return (FALSE);
    if (in_array == (iarray) NULL) return(FALSE);
    if (out_array == (iarray) NULL) return(FALSE);
    if (iarray_num_dim(in_array) != 2) return(FALSE);
    if (iarray_num_dim(out_array) != 2) return(FALSE);
    if (iarray_type(in_array) != K_BYTE) return(FALSE);
    if (iarray_type(out_array) != K_BYTE) return(FALSE);
    xlen = iarray_dim_length (in_array, 1); 
    ylen = iarray_dim_length (in_array, 0);
    out_xlen = iarray_dim_length (out_array, 1); 
    out_ylen = iarray_dim_length (out_array, 0);
    if (tmp_array == NULL)
    {
	if ( ( tmp_array = iarray_create_2D (out_ylen, out_xlen, K_INT) )
	    == NULL )
	{
	    m_abort (function_name, "working array");
	}
    }
    else if ( (iarray_dim_length (tmp_array, 1) < out_xlen) ||
	     (iarray_dim_length (tmp_array, 0) < out_ylen) )
    {
	iarray_dealloc (tmp_array);
	if ( ( tmp_array = iarray_create_2D (out_ylen, out_xlen, K_INT) )
	    == NULL )
	{
	    m_abort (function_name, "working array");
	}
    }
    if (xlen*exp_fac > out_xlen) return (FALSE);
    if (ylen*exp_fac > out_ylen) return (FALSE);

    y = mal_vector(-1, 2);	/* create work arrays */
    w = mal_matrix(1, exp_fac-1, -1, 2);
  
    setup_interpol(w, exp_fac);	/* setup interpolation coefficients */
  
    up_x = xlen - 2;
    up_y = ylen;

    for (j = 0; j < up_y; j++) { /* first loop: interpolate in X */
	yco = exp_fac*j;

	y[0]  = B2 (in_array, j, 0); /* handle left edge */
	y[1]  = B2 (in_array, j, 1);
	y[2]  = B2 (in_array, j, 2);
	switch (edge_method) {	/* how to treat edges */
	  case INTER_EDGE_SAME : {
	      y[-1] = y[0];	/* same as edge data */
	      break;
	  } 
	  case INTER_EDGE_MIRROR : { /* mirror image in X */
	      y[-1] = B2(in_array, j, xlen-1);
	      break;
	  }
	  case INTER_EDGE_ZERO : /* set to zero */
	  default : {
	      y[-1] = 0.0;
	      break;
	  }
	}
	/* do not interpolate where you don't need */
	I2 (tmp_array, yco, 0) = y[0];
	for (ii = 1; ii < exp_fac; ii++) { /* interpolate */
	    I2 (tmp_array, yco, ii) = interpol(y, w, ii);
	} /* and store in outout image */
    
	for (k = 1; k < up_x; k++) { /* central part */
	    y[-1] = B2(in_array, j, k-1);
	    y[0]  = B2(in_array, j, k);
	    y[1]  = B2(in_array, j, k+1);
	    y[2]  = B2(in_array, j, k+2);
      
	    xco = exp_fac*k;

	    I2 (tmp_array, yco, xco) = y[0];
	    for (ii = 1; ii < exp_fac; ii++) {
		I2 (tmp_array, yco, xco+ii) = interpol(y, w, ii);
	    }
	}

	y[-1] = B2(in_array, j, xlen - 3); /* first right  edge of image */
	y[0]  = B2(in_array, j, xlen - 2);
	y[1]  = B2(in_array, j, xlen - 1);

	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[2] = B2(in_array, j, 0);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = 0.0;
	      break;
	  }
	}
      
	xco = exp_fac*(xlen-2);

	I2 (tmp_array, yco, xco) = y[0];
	for (ii = 1; ii < exp_fac; ii++) {
	    I2 (tmp_array, yco, xco+ii) = interpol(y, w, ii);
	}
    

	y[-1] = B2(in_array, j, xlen - 2); /* second right  edge of image */
	y[0]  = B2(in_array, j, xlen - 1);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1] = y[0];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[1] = B2(in_array, j, 0);
	      y[2] = B2(in_array, j, 1);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = y[1] = 0.0;
	      break;
	  }
	}

	xco = exp_fac*(xlen-1);

	I2 (tmp_array, yco, xco) = y[0];
	for (ii = 1; ii < exp_fac; ii++) {
	    I2 (tmp_array, yco, xco+ii) = interpol(y, w, ii);
	}
    
    }

    up_y = exp_fac*(ylen-2);
    up_x = exp_fac*xlen;
    /* second loop: interpolate in Y */
    for (k = 0; k < up_x; k++) { /* use data stored output image  */
	y[0]  = I2 (tmp_array, 0, k); /* bottom edge */
	y[1]  = I2 (tmp_array, exp_fac, k);
	y[2]  = I2 (tmp_array, 2*exp_fac, k);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[-1] = y[0];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[-1] = I2 (tmp_array, out_ylen-1, k);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[-1] = 0.0;
	      break;
	  }
	}


	for (ii = 1; ii < exp_fac; ii++) {
	    I2 (tmp_array, ii, k) = interpol(y, w, ii);
	}
    }

    for (j = exp_fac; j < up_y; j  += exp_fac) { /* main part of image */
	for (k = 0; k < up_x; k++) {
	    y[-1] = I2 (tmp_array, j-exp_fac, k);
	    y[0]  = I2 (tmp_array, j, k);
	    y[1]  = I2 (tmp_array, j+exp_fac, k);
	    y[2]  = I2 (tmp_array, j+2*exp_fac, k);

	    for (ii = 1; ii < exp_fac; ii++) {
		I2 (tmp_array, j+ii, k) = interpol(y, w, ii);
	    }
	}
    }

    yco = up_y;
  
    for (k = 0; k < up_x; k++) { /* first top row */
	y[-1] = I2 (tmp_array, yco - exp_fac, k);
	y[0]  = I2 (tmp_array, yco          , k);
	y[1]  = I2 (tmp_array, yco + exp_fac, k);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[2] = I2 (tmp_array, 0, k);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = 0.0;
	      break;
	  }
	}

	for (ii = 1; ii < exp_fac; ii++) {
	    I2 (tmp_array, yco+ii, k) = interpol(y, w, ii);
	}
    }

    yco = yco + exp_fac;
    for (k = 0; k < up_x; k++) { /* second top row */
	y[-1] = I2 (tmp_array, yco - exp_fac, k);
	y[0]  = I2 (tmp_array, yco, k);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1] = y[0];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[1] = I2 (tmp_array, 0, k);
	      y[2] = I2 (tmp_array, exp_fac, k);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = y[1] = 0.0;
	      break;
	  }
	}

	for (ii = 1; ii < exp_fac; ii++) {
	    I2 (tmp_array, yco+ii, k) = interpol(y, w, ii);
	}
    }
    /*  Copy working array to output  */
    for (k = 0; k < out_ylen; ++k) for (j = 0; j < out_xlen; ++j)
    {
	B2 (out_array, k, j) = I2 (tmp_array, k, j);
    }
    free_vector(y, -1, 2);	/* clean up */
    free_matrix(w, 1, exp_fac-1, -1, 2);
  
    return(TRUE);
}

flag iarray_expand_ubyte (iarray in_array, iarray out_array, int exp_fac,
			  int edge_method)
/*  
   Expands an 2d iarray with an integral factor, using anti aliasing
   interpolation. Returns TRUE on succes, else FALSE. 

   iarray    in_array      input image
   iarray    out_array     output image
   int       exp_fac       expand factor for image
   int       edge_method   defines how edges are handled

   Updates:    13-Nov-94   Created from old code
   15-Nov-94   Added handling of edges
*/
{
              
    int     j, k, xlen, ylen, out_xlen, out_ylen;
    int     ii, xco, yco, up_x, up_y;
    float   *y, **w;
    static char function_name[] = "iarray_expand_ubyte";

    if (exp_fac < 2) return(FALSE);
    if (in_array == (iarray) NULL) return(FALSE);
    if (out_array == (iarray) NULL) return(FALSE);
    if (iarray_num_dim(in_array) != 2) return(FALSE);
    if (iarray_num_dim(out_array) != 2) return(FALSE);
    if (iarray_type(in_array) != K_UBYTE) return(FALSE);
    if (iarray_type(out_array) != K_UBYTE) return(FALSE);
    xlen = iarray_dim_length(in_array, 1); 
    ylen = iarray_dim_length(in_array, 0);
    out_xlen = iarray_dim_length(out_array, 1); 
    out_ylen = iarray_dim_length(out_array, 0);
    if (tmp_array == NULL)
    {
	if ( ( tmp_array = iarray_create_2D (out_ylen, out_xlen, K_INT) )
	    == NULL )
	{
	    m_abort (function_name, "working array");
	}
    }
    else if ( (iarray_dim_length (tmp_array, 1) < out_xlen) ||
	     (iarray_dim_length (tmp_array, 0) < out_ylen) )
    {
	iarray_dealloc (tmp_array);
	if ( ( tmp_array = iarray_create_2D (out_ylen, out_xlen, K_INT) )
	    == NULL )
	{
	    m_abort (function_name, "working array");
	}
    }

    if (xlen*exp_fac > out_xlen) {
	return(FALSE);
    }
    if (ylen*exp_fac > out_ylen) {
	return(FALSE);
    }

    y = mal_vector(-1, 2);	/* create work arrays */
    w = mal_matrix(1, exp_fac-1, -1, 2);
  
    setup_interpol(w, exp_fac);	/* setup interpolation coefficients */
  
    up_x = xlen - 2;
    up_y = ylen;

    for (j = 0; j < up_y; j++) { /* first loop: interpolate in X */
	yco = exp_fac*j;

	y[0]  = UB2(in_array, j, 0); /* handle left edge */
	y[1]  = UB2(in_array, j, 1);
	y[2]  = UB2(in_array, j, 2);
	switch (edge_method) {	/* how to treat edges */
	  case INTER_EDGE_SAME : {
	      y[-1] = y[0];	/* same as edge data */
	      break;
	  } 
	  case INTER_EDGE_MIRROR : { /* mirror image in X */
	      y[-1] = UB2(in_array, j, xlen-1);
	      break;
	  }
	  case INTER_EDGE_ZERO : /* set to zero */
	  default : {
	      y[-1] = 0.0;
	      break;
	  }
	}
 
	I2 (tmp_array, yco, 0) = y[0]; /* do not interpolate where you don't need */
	for (ii = 1; ii < exp_fac; ii++) { /* interpolate */
	    I2 (tmp_array, yco, ii) = interpol(y, w, ii);
	} /* and store in outout image */
    
	for (k = 1; k < up_x; k++) { /* central part */
	    y[-1] = UB2(in_array, j, k-1);
	    y[0]  = UB2(in_array, j, k);
	    y[1]  = UB2(in_array, j, k+1);
	    y[2]  = UB2(in_array, j, k+2);
      
	    xco = exp_fac*k;

	    I2 (tmp_array, yco, xco) = y[0];
	    for (ii = 1; ii < exp_fac; ii++) {
		I2 (tmp_array, yco, xco+ii) = interpol(y, w, ii);
	    }
	}

	y[-1] = UB2(in_array, j, xlen - 3); /* first right  edge of image */
	y[0]  = UB2(in_array, j, xlen - 2);
	y[1]  = UB2(in_array, j, xlen - 1);

	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[2] = UB2(in_array, j, 0);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = 0.0;
	      break;
	  }
	}
      
	xco = exp_fac*(xlen-2);

	I2 (tmp_array, yco, xco) = y[0];
	for (ii = 1; ii < exp_fac; ii++) {
	    I2 (tmp_array, yco, xco+ii) = interpol(y, w, ii);
	}
    

	y[-1] = UB2(in_array, j, xlen - 2); /* second right  edge of image */
	y[0]  = UB2(in_array, j, xlen - 1);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1] = y[0];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[1] = UB2(in_array, j, 0);
	      y[2] = UB2(in_array, j, 1);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = y[1] = 0.0;
	      break;
	  }
	}

	xco = exp_fac*(xlen-1);

	I2 (tmp_array, yco, xco) = y[0];
	for (ii = 1; ii < exp_fac; ii++) {
	    I2 (tmp_array, yco, xco+ii) = interpol(y, w, ii);
	}
    
    }

    up_y = exp_fac*(ylen-2);
    up_x = exp_fac*xlen;
    /* second loop: interpolate in Y */
    for (k = 0; k < up_x; k++) { /* use data stored output image  */
	y[0]  = I2 (tmp_array, 0, k); /* bottom edge */
	y[1]  = I2 (tmp_array, exp_fac, k);
	y[2]  = I2 (tmp_array, 2*exp_fac, k);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[-1] = y[0];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[-1] = I2 (tmp_array, out_ylen-1, k);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[-1] = 0.0;
	      break;
	  }
	}


	for (ii = 1; ii < exp_fac; ii++) {
	    I2 (tmp_array, ii, k) = interpol(y, w, ii);
	}
    }

    for (j = exp_fac; j < up_y; j  += exp_fac) { /* main part of image */
	for (k = 0; k < up_x; k++) {
	    y[-1] = I2 (tmp_array, j-exp_fac, k);
	    y[0]  = I2 (tmp_array, j, k);
	    y[1]  = I2 (tmp_array, j+exp_fac, k);
	    y[2]  = I2 (tmp_array, j+2*exp_fac, k);

	    for (ii = 1; ii < exp_fac; ii++) {
		I2 (tmp_array, j+ii, k) = interpol(y, w, ii);
	    }
	}
    }

    yco = up_y;
  
    for (k = 0; k < up_x; k++) { /* first top row */
	y[-1] = I2 (tmp_array, yco - exp_fac, k);
	y[0]  = I2 (tmp_array, yco          , k);
	y[1]  = I2 (tmp_array, yco + exp_fac, k);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[2] = I2 (tmp_array, 0, k);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = 0.0;
	      break;
	  }
	}

	for (ii = 1; ii < exp_fac; ii++) {
	    I2 (tmp_array, yco+ii, k) = interpol(y, w, ii);
	}
    }

    yco = yco + exp_fac;
    for (k = 0; k < up_x; k++) { /* second top row */
	y[-1] = I2 (tmp_array, yco - exp_fac, k);
	y[0]  = I2 (tmp_array, yco, k);
	switch (edge_method) {
	  case INTER_EDGE_SAME : {
	      y[2] = y[1] = y[0];
	      break;
	  } 
	  case INTER_EDGE_MIRROR : {
	      y[1] = I2 (tmp_array, 0, k);
	      y[2] = I2 (tmp_array, exp_fac, k);
	      break;
	  }
	  case INTER_EDGE_ZERO :
	  default : {
	      y[2] = y[1] = 0.0;
	      break;
	  }
	}

	for (ii = 1; ii < exp_fac; ii++) {
	    I2 (tmp_array, yco+ii, k) = interpol(y, w, ii);
	}
    }
    /*  Copy working array to output  */
    for (k = 0; k < out_ylen; ++k) for (j = 0; j < out_xlen; ++j)
    {
	ii = I2 (tmp_array, k, j);
	if (ii < 0) ii = 0;
	if (ii > 255) ii = 255;
	UB2 (out_array, k, j) = ii;
    }
    free_vector(y, -1, 2);	/* clean up */
    free_matrix(w, 1, exp_fac-1, -1, 2);
    return(TRUE);
}


void setup_interpol(w, expa)
  float   **w;
  int     expa;
{
/*  
  set up of interpolation coefficients for interpol for B(0,0.5) anti aliasing
  polynomial.
  
  float   **w   array to contain coefficients
  int     expa  expanding factor for image 

  Updates   13-Nov-94:    created
*/    
  int     i;
  float   x, x2, x3;
  
  for (i = 1; i < expa; i++) {
    x = ((float) i) / ((float) expa);
    x2 = x*x;
    x3 = x2*x;
    
    w[i][-1] = x2-0.5*(x3+x);
    w[i][0]  = 1.5*x3-2.5*x2+1.0;
    w[i][1]  = 2.0*x2-1.5*x3+0.5*x;
    w[i][2]  = 0.5*(x3-x2);
  }
}


float interpol(y, w, ind)
  float *y, **w;
  int   ind;
{
/*
  interpolates array using pre-computed coefficients
  
  float *y    array[-1,2] with values;
  float **w   array[1,expa-1][-1,2] with coefficients
  int   int   specifies which point to take: x value is ind/expa
  
  Updates   13-Nov-94:   created
*/  
    
  return(w[ind][-1]*y[-1]+w[ind][0]*y[0]+w[ind][1]*y[1]+w[ind][2]*y[2]);
}   

#undef    INTERPOL


