/*  globals.c

    Source file for  xray  (Karma volume rendering tool).

    Copyright (C) 1993-1999  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*  This file contains all global variables.


    Written by      Richard Gooch   26-DEC-1994

    Last updated by Richard Gooch   10-SEP-1999


*/
#include <stdio.h>
#include <math.h>
#include <karma.h>
#include <karma_viewimg.h>
#include <karma_overlay.h>
#include <karma_wcs.h>
#include "xraycast_3d.h"


/*  Public data  */

/*  The cube  */
iarray unfiltered_cube = NULL;
iarray filtered_cube = NULL;
iarray byte_cube = NULL;
double byte_cube_scale = 1.0;
double byte_cube_offset = 0.0;
unsigned int raycast_mode = MODE_NORMAL;
flag byte_cube_valid = FALSE;
KwcsAstro astro_projection = NULL;

/*  Orientation display  */
KPixCanvas orientation_pixcanvas = NULL;
KWorldCanvas orientation_worldcanvas = NULL;
KOverlayList cyclops_olist = NULL;
KOverlayList left_olist = NULL;
KOverlayList right_olist = NULL;

/*  Substance display  */
KPixCanvas substance_pixcanvas = NULL;

/*  SpaceBall information  */
flag using_spaceball = FALSE;
float spaceball_sensitivity = 1.0;

/*  Continuous Hot Gas information  */
KPixCanvas cont_hg_hist_pixcanvas = NULL;
int cont_hg_blank_min = -128;
int cont_hg_blank_max = -128;
int cont_hg_alpha_tenths = 10;

/*  Three colour Hot Gas information  */
int threecolour_hg_section1_red_intensity = INTENSITY_GRADATIONS;
int threecolour_hg_section1_green_intensity = 0;
int threecolour_hg_section1_blue_intensity = 0;
int threecolour_hg_section1_intensity_alpha_tenths = 10;
int threecolour_hg_section1_opacity_alpha_tenths = 10;
int threecolour_hg_section1_opacity_peak = 100;
int threecolour_hg_section1_end = -40;
int threecolour_hg_section2_red_intensity = 0;
int threecolour_hg_section2_green_intensity = INTENSITY_GRADATIONS;
int threecolour_hg_section2_blue_intensity = 0;
int threecolour_hg_section2_intensity_alpha_tenths = 10;
int threecolour_hg_section2_opacity_alpha_tenths = 10;
int threecolour_hg_section2_opacity_peak = 100;
int threecolour_hg_section2_end = 40;
int threecolour_hg_section3_red_intensity = 0;
int threecolour_hg_section3_green_intensity = 0;
int threecolour_hg_section3_blue_intensity = INTENSITY_GRADATIONS;
int threecolour_hg_section3_intensity_alpha_tenths = 10;
int threecolour_hg_section3_opacity_alpha_tenths = 10;
int threecolour_hg_section3_opacity_peak = 100;
int threecolour_hg_blank_min = -128;
int threecolour_hg_blank_max = -128;

/*  Miscellaneous  */
flag hide_wireframe = FALSE;
long compute_time = 0;

/*  Stereo control  */
flag stereo = FALSE;

/*  Mono Hot Gas information  */
KPixCanvas mono_hg_pixcanvas = NULL;
int mono_hg_blank_min = -128;
int mono_hg_blank_max = -128;
int mono_hg_intensity_alpha_tenths = 10;
int mono_hg_opacity_alpha_tenths = 10;


/*  Rotation specification globals  */
rotation_specification rotation = {0.0, 0.0, 0.0, (float) DEFAULT_DISTANCE};

/*  3 dimensional cursor globals  */
Kcoord_3d cursor3d = {0.0, 0.0, 0.0};     /*  Voxel co-ordinates  */
Kcoord_3d corner3d = {-1.0, -1.0, -1.0};  /*  Voxel co-ordinates  */
unsigned int cursor_3d_mode = CURSOR_3D_NONE;
int sphere_radius = 25;

/*  Sub-cube specification ratios  */
i_coord_3d subcube_start = {0, 0, 0};
i_coord_3d subcube_end = {0, 0, 0};

/*  Substance information  */
flag substances_needed = FALSE;

/*  Histogram information  */
unsigned int histogram_peak = 0;
int histogram_mode = 0;
KOverlayList histogram_olist = NULL;

/*  Image Display  */
KWorldCanvas cyclops_image_pc_worldcanvas = NULL;
KWorldCanvas cyclops_image_rgb_worldcanvas = NULL;
KWorldCanvas left_image_pc_worldcanvas = NULL;
KWorldCanvas right_image_pc_worldcanvas = NULL;
KWorldCanvas left_image_rgb_worldcanvas = NULL;
KWorldCanvas right_image_rgb_worldcanvas = NULL;
KWorldCanvas visible_worldcanvas = NULL;
flag inhibit_images = FALSE;

/*  Computed image parameters (prior to expansion)  */
unsigned int raw_image_width = 0;
unsigned int raw_image_height = 0;
/*  Raw image buffers. These are only used if the image is expanded  */
char *raw_left_buffer = NULL;
char *raw_right_buffer = NULL;

/*  Computed image parameters (after expansion)  */
unsigned int exp_image_width = 0;
unsigned int exp_image_height = 0;
array_desc *exp_arr_desc = NULL;

/*  Computed image/movie buffers. These buffers contain the expanded image(s),
    and are large enough to contain a movie. They never shrink. When not
    displaying stereo, the left buffer is used, and the right buffer may be
    NULL. Note that the right buffer is private to display.c  */
char *movie_left_buffer = NULL;
unsigned int movie_buf_len = 0;
double movie_min = TOOBIG;
double movie_max = -TOOBIG;

/*  ViewableImage list  */
ViewableImage vimage_left = NULL;
ViewableImage vimage_right = NULL;
ViewableImage *vimages_left = NULL;
ViewableImage *vimages_right = NULL;
unsigned int vimage_buf_len = 0;
unsigned int num_frames = 0;


/*  Movie control  */
flag running_movie = FALSE;
int num_frames_to_compute = 0;

/*  Ray display  */
KOverlayList ray_display_olist = NULL;

/*  Shader  */
CONST char *current_shader_name = NULL;


/*  Anti-aliasing hack  */
int image_expand_factor = 1;

/*  The volume rendering contexts  */
KVolumeRenderContext main_vrend_ctx = NULL;

/*  Debug control  */
flag show_timings = FALSE;

/*  Depth shader information  */
uaddr offset_pixel1;
uaddr offset_pixel2;
uaddr offset_pixel3;
uaddr offset_depth;
uaddr offset_cumulative_opacity;
uaddr offset_opacity_rolloff;
uaddr offset_depth_offset;
uaddr offset_depth_factor;

/*  Cut plane information  */
int cut_plane_depth = 50;
flag depth_plane_valid = FALSE;

/*  2 dimensional cursor globals  */
unsigned int cursor_2d_mode = CURSOR_2D_DUMB;

/*  File segment information  */
segment_list *first_segment_list = NULL;
