/*  region_xt.c

    Source file for  xray  (Karma volume rendering tool).

    Copyright (C) 1998-1999  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*
    This file contains the code necessary for creating the region controls.


    Written by      Richard Gooch   28-JUL-1998

    Updated by      Richard Gooch   21-AUG-1998: Completed first basic version
  with save button.

    Updated by      Richard Gooch   31-AUG-1998: Added dynamic creation of
  render popup.

    Updated by      Richard Gooch   3-SEP-1998: Support multiple export formats

    Updated by      Richard Gooch   9-SEP-1998: Added "Apply Subcube" button.

    Updated by      Richard Gooch   12-SEP-1998: Added "Sum" button.

    Updated by      Richard Gooch   27-OCT-1998: Took account of change to
  <create_vrender_context>.

    Last updated by Richard Gooch   19-MAR-1999: Took account of change to
  <create_vrender_context>.


*/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Dialog.h>
#include <karma.h>
#ifndef X11
#  define X11
#endif
#include <k_event_codes.h>
#include <karma_foreign.h>
#include <karma_xtmisc.h>
#include <karma_iarray.h>
#include <karma_ch.h>
#include <Xkw/Dialogpopup.h>
#include <Xkw/ChoiceMenu.h>
#include "RenderDisplayPopup.h"
#include "xraycast_3d.h"


typedef struct
{
    i_coord_3d start;
    i_coord_3d stop;    /*  End position plus 1  */
    i_coord_3d length;
} subcube_type;


/*  Private functions  */
STATIC_FUNCTION (Widget create_region_win,
		 (Widget main_shell, Widget popup_btn) );
STATIC_FUNCTION (flag region_prerender, (KVolumeRenderContext vrend_ctx) );
STATIC_FUNCTION (void lock_corner_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (void clear_corner_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (void format_menu_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (flag get_subcube, (subcube_type *subcube, flag verbose) );
STATIC_FUNCTION (void save_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (void apply_subcube_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (void sum_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );


/*  Private data  */

#define FORMAT_KARMA   0
#define FORMAT_FITS    1
#define FORMAT_MIRIAD  2
#define FORMAT_GIPSY   3
#define NUM_FORMATS    4

static char *format_choices[NUM_FORMATS] =
{
    "Karma", "FITS", "Miriad", "GIPSY",
};
static unsigned int format = FORMAT_KARMA;


/*  Public functions follow  */

void region_button_cbk (Widget w, XtPointer client_data, XtPointer call_data)
/*  This is the region button callback.
*/
{
    static Widget popup = NULL;

    if (popup != NULL) return;
    popup = create_region_win ( (Widget) client_data, w );
}   /*  End Function region_button_cbk  */


/*  Private functions follow  */

static Widget create_region_win (Widget main_shell, Widget popup_btn)
{
    KVolumeRenderContext region_vrend_ctx;
    Widget popup, form, w, save_dialog_popup, region_render_popup;
    Widget lock_btn, clear_btn, save_region_btn;
    extern flag verbose;
    extern Widget image_display;
    /*static char function_name[] = "create_region_win";*/

    region_vrend_ctx = create_vrender_context (verbose, FALSE);
    region_render_popup =
	XtVaCreatePopupShell ("regionSubvolume", renderDisplayPopupWidgetClass,
			      image_display,
			      XkwNvolumeRenderContext, region_vrend_ctx,
			      XkwNpreRenderFunction, region_prerender,
			      NULL);
    popup = XtVaCreatePopupShell ("region_winpopup",
				  topLevelShellWidgetClass, main_shell,
				  XtNtitle, "Region Control Window",
				  NULL);
    XtAddCallback (popup_btn, XtNcallback, xtmisc_popup_cbk, popup);
    XtAddCallback (popup_btn, XtNcallback, xtmisc_popup_cbk,
		   region_render_popup);
    w = XtVaCreatePopupShell ("savePopup", dialogpopupWidgetClass, main_shell,
			      XtNlabel, "Output subcube         ",
			      XkwNextension, "-----------",
			      NULL);
    save_dialog_popup = w;
    XtAddCallback (w, XtNcallback, save_cbk, NULL);
    form = XtVaCreateManagedWidget ("form", formWidgetClass, popup,
				    XtNborderWidth, 0,
				    NULL);
    w = XtVaCreateManagedWidget ("closeButton", commandWidgetClass, form,
				 XtNlabel, "Close",
				 NULL);
    XtAddCallback (w, XtNcallback, xtmisc_popdown_cbk, popup);
    XtAddCallback (w, XtNcallback, xtmisc_popdown_cbk, region_render_popup);
    XtAddCallback (w, XtNcallback, xtmisc_popdown_cbk, save_dialog_popup);
    w = XtVaCreateManagedWidget ("button", commandWidgetClass, form,
				 XtNlabel, "Lock First Corner",
				 XtNfromHoriz, w,
				 NULL);
    lock_btn = w;
    XtAddCallback (w, XtNcallback, lock_corner_cbk, NULL);
    w = XtVaCreateManagedWidget ("button", commandWidgetClass, form,
				 XtNlabel, "Clear Corner",
				 XtNfromHoriz, w,
				 NULL);
    clear_btn = w;
    XtAddCallback (w, XtNcallback, clear_corner_cbk, NULL);
    XtAddCallback (w, XtNcallback, xtmisc_popdown_cbk, save_dialog_popup);
    w = XtVaCreateManagedWidget ("menuButton", choiceMenuWidgetClass, form,
				 XtNlabel, "Save",
				 XkwNmenuTitle, "Save Menu",
				 XtNmenuName, "saveMenu",
				 XkwNnumItems, NUM_FORMATS,
				 XkwNitemStrings, format_choices,
				 XtNfromVert, w,
				 XtNsensitive, FALSE,
				 NULL);
    XtAddCallback (w, XkwNselectCallback, format_menu_cbk, save_dialog_popup);
    save_region_btn = w;
    XtAddCallback (w, XtNcallback, format_menu_cbk, save_dialog_popup);
    XtAddCallback (w, XtNcallback, xtmisc_popup_cbk, save_dialog_popup);
    XtAddCallback (lock_btn, XtNcallback, xtmisc_sensitive_cbk,
		   save_region_btn);
    XtAddCallback (clear_btn, XtNcallback, xtmisc_insensitive_cbk,
		   save_region_btn);
    w = XtVaCreateManagedWidget ("button", commandWidgetClass, form,
				 XtNlabel, "Apply Subcube",
				 XtNfromVert, clear_btn,
				 XtNfromHoriz, w,
				 XtNsensitive, FALSE,
				 NULL);
    XtAddCallback (lock_btn, XtNcallback, xtmisc_sensitive_cbk, w);
    XtAddCallback (clear_btn, XtNcallback, xtmisc_insensitive_cbk, w);
    XtAddCallback (w, XtNcallback, apply_subcube_cbk, clear_btn);
    w = XtVaCreateManagedWidget ("button", commandWidgetClass, form,
				 XtNlabel, "Sum",
				 XtNfromVert, clear_btn,
				 XtNfromHoriz, w,
				 XtNsensitive, FALSE,
				 NULL);
    XtAddCallback (lock_btn, XtNcallback, xtmisc_sensitive_cbk, w);
    XtAddCallback (clear_btn, XtNcallback, xtmisc_insensitive_cbk, w);
    XtAddCallback (w, XtNcallback, sum_cbk, NULL);
    XtPopup (popup, XtGrabNone);
    XtPopup (region_render_popup, XtGrabNone);
    XtRealizeWidget (save_dialog_popup);  /*  To get the extension geometry  */
    return (popup);
}   /*  End Function create_region_win  */

static flag region_prerender (KVolumeRenderContext vrend_ctx)
/*  [SUMMARY] Compute information prior to rendering for the region display.
    <vrend_ctx> The KVolumeRenderContext.
    [RETURNS] TRUE on success, else FALSE indicating rendering should be
    aborted.
*/
{
    subcube_type sc;
    extern Kcoord_3d corner3d;
    extern rotation_specification rotation;

    if (corner3d.x < 0.0) return (FALSE);
    if ( !get_subcube (&sc, FALSE) ) return (FALSE);
    vrender_set_context_attributes
	(vrend_ctx,
	 VRENDER_CONTEXT_ATT_SUBCUBE_X_START, sc.start.x,
	 VRENDER_CONTEXT_ATT_SUBCUBE_Y_START, sc.start.y,
	 VRENDER_CONTEXT_ATT_SUBCUBE_Z_START, sc.start.z,
	 VRENDER_CONTEXT_ATT_SUBCUBE_X_END, sc.stop.x - 1,
	 VRENDER_CONTEXT_ATT_SUBCUBE_Y_END, sc.stop.y - 1,
	 VRENDER_CONTEXT_ATT_SUBCUBE_Z_END, sc.stop.z - 1,
	 VRENDER_CONTEXT_ATT_END);
    /*  Since the view depends on the subcube, must update view here  */
    generic_rotation_callback (vrend_ctx, NULL, &rotation, NULL);
    return (TRUE);
}   /*  End Function region_prerender  */

static void lock_corner_cbk (Widget w, XtPointer client_data,
			     XtPointer call_data)
/*  This is the lock corner callback.
*/
{
    extern Kcoord_3d cursor3d, corner3d;

    corner3d = cursor3d;
    fprintf (stderr, "Region corner at %.1f  %.1f  %.1f\n",
	     corner3d.x, corner3d.y, corner3d.z);
}   /*  End Function lock_corner_cbk   */

static void clear_corner_cbk (Widget w, XtPointer client_data,
			      XtPointer call_data)
/*  This is the clear corner callback.
*/
{
    extern Kcoord_3d corner3d;

    corner3d.x = -1.0;
    corner3d.y = -1.0;
    corner3d.z = -1.0;
}   /*  End Function clear_corner_cbk   */

static void format_menu_cbk (Widget w, XtPointer client_data,
			     XtPointer call_data)
/*  This is the format change callback.
*/
{
    Widget save_dialog_popup = client_data;

    format = *(int *) call_data;
    switch (format)
    {
      case FORMAT_KARMA:
	XtVaSetValues (save_dialog_popup, XkwNextension, ".kf", NULL);
	break;
      case FORMAT_FITS:
	XtVaSetValues (save_dialog_popup, XkwNextension, ".fits", NULL);
	break;
      case FORMAT_MIRIAD:
      case FORMAT_GIPSY:
	XtVaSetValues (save_dialog_popup, XkwNextension, "", NULL);
	break;
    }
    XtPopup (save_dialog_popup, XtGrabNone);
}   /*  End Function format_menu_cbk  */

static flag get_subcube (subcube_type *sc, flag verbose)
/*  [SUMMARY] Get the subcube information.
    <sc> The subcube information is written here.
    <verbose> If TRUE a warning message is display for too small cubes.
    [RETURNS] TRUE if the subcube is valid, else FALSE.
*/
{
    int tmp;
    extern Kcoord_3d cursor3d, corner3d;

    sc->start.x = cursor3d.x;
    sc->stop.x = corner3d.x;
    if (sc->start.x > sc->stop.x) swap (sc->start.x, sc->stop.x, tmp);
    ++sc->stop.x;
    sc->length.x = sc->stop.x - sc->start.x;
    if (sc->length.x < 2)
    {
	if (verbose) fprintf (stderr, "xlength: %d too small\n", sc->length.x);
	return (FALSE);
    }
    sc->start.y = cursor3d.y;
    sc->stop.y = corner3d.y;
    if (sc->start.y > sc->stop.y) swap (sc->start.y, sc->stop.y, tmp);
    ++sc->stop.y;
    sc->length.y = sc->stop.y - sc->start.y;
    if (sc->length.y < 2)
    {
	if (verbose) fprintf (stderr, "ylength: %d too small\n", sc->length.y);
	return (FALSE);
    }
    sc->start.z = cursor3d.z;
    sc->stop.z = corner3d.z;
    if (sc->start.z > sc->stop.z) swap (sc->start.z, sc->stop.z, tmp);
    ++sc->stop.z;
    sc->length.z = sc->stop.z - sc->start.z;
    if (sc->length.z < 2)
    {
	if (verbose) fprintf (stderr, "zlength: %d too small\n", sc->length.z);
	return (FALSE);
    }
    return (TRUE);
}   /*  End Function get_subcube  */

static void save_cbk (Widget w, XtPointer client_data, XtPointer call_data)
/*  This is the save callback.
*/
{
    Channel ch;
    flag ok;
    iarray cube, alias, subcube;
    subcube_type sc;
    String fname = (String) call_data;
    extern iarray byte_cube, unfiltered_cube;
    static char function_name[] = "save_cbk";

    if ( !get_subcube (&sc, TRUE) ) return;
    cube = unfiltered_cube;
    if (cube == NULL) cube = byte_cube;
    if ( ( alias =
	   iarray_get_sub_array_3D (cube, sc.start.z, sc.start.y, sc.start.x,
				    sc.length.z, sc.length.y, sc.length.x) )
	 == NULL )
    {
	fprintf (stderr, "%s: error creating subcube alias\n", function_name);
	return;
    }
    subcube = iarray_create_from_template (alias, iarray_type (alias),
					   TRUE, TRUE, TRUE);
    if (subcube == NULL)
    {
	fprintf (stderr, "%s: error creating subcube\n", function_name);
	iarray_dealloc (alias);
	return;
    }
    ok = iarray_copy_data (subcube, alias, FALSE);
    iarray_dealloc (alias);
    if (!ok)
    {
	fprintf (stderr, "%s: error copying subcube\n", function_name);
	iarray_dealloc (subcube);
	return;
    }
    /*  Now write  */
    ok = FALSE;
    switch (format)
    {
      case FORMAT_KARMA:
	ok = iarray_write (subcube, fname);
	break;
      case FORMAT_FITS:
	if ( ( ch = ch_open_file (fname, "w") ) == NULL )
	{
	    fprintf (stderr, "%s: error opening file\t%s\n",
		     function_name, fname);
	    break;
	}
	ok = foreign_fits_write_iarray (ch, subcube, FA_FITS_WRITE_END);
	if (ok) ok = ch_close (ch);
	else
	{
	    ch_close (ch);
	    unlink (fname);
	}
	break;
      case FORMAT_MIRIAD:
	ok = foreign_miriad_write_iarray (fname, subcube, FA_MIRIAD_WRITE_END);
	break;
      case FORMAT_GIPSY:
	ok = foreign_gipsy_write_iarray (fname, subcube, FA_GIPSY_WRITE_END);
	break;
    }
    iarray_dealloc (subcube);
    if (ok) fprintf (stderr, "Wrote subcube: \"%s\"\n", fname);
    else fprintf (stderr, "Error writing subcube\n");
}   /*  End Function save_cbk   */

static void apply_subcube_cbk (Widget w, XtPointer client_data,
			       XtPointer call_data)
/*  This is the apply subcube callback.
*/
{
    subcube_type sc;
    Widget clear_btn = (Widget) client_data;
    extern Widget start_x_sld, end_x_sld, start_y_sld, end_y_sld;
    extern Widget start_z_sld, end_z_sld;

    if ( !get_subcube (&sc, TRUE) ) return;
    XtVaSetValues (start_x_sld, XtNvalue, sc.start.x, NULL);
    XtVaSetValues (start_y_sld, XtNvalue, sc.start.y, NULL);
    XtVaSetValues (start_z_sld, XtNvalue, sc.start.z, NULL);
    XtVaSetValues (end_x_sld, XtNvalue, sc.stop.x - 1, NULL);
    XtVaSetValues (end_y_sld, XtNvalue, sc.stop.y - 1, NULL);
    XtVaSetValues (end_z_sld, XtNvalue, sc.stop.z - 1, NULL);
    XtCallCallbacks (clear_btn, XtNcallback, NULL);
    register_subcube_size ();
}   /*  End Function apply_subcube_cbk   */

static void sum_cbk (Widget w, XtPointer client_data, XtPointer call_data)
/*  This is the sum callback.
*/
{
    flag ok;
    iarray cube, subcube;
    subcube_type sc;
    double scale, offset, num;
    double sum[2];
    char txt[STRING_LENGTH];
    extern iarray byte_cube, unfiltered_cube;
    static char function_name[] = "sum_cbk";

    if ( !get_subcube (&sc, TRUE) ) return;
    cube = unfiltered_cube;
    if (cube == NULL) cube = byte_cube;
    if ( ( subcube =
	   iarray_get_sub_array_3D (cube, sc.start.z, sc.start.y, sc.start.x,
				    sc.length.z, sc.length.y, sc.length.x) )
	 == NULL )
    {
	fprintf (stderr, "%s: error creating subcube alias\n", function_name);
	return;
    }
    sum[0] = 0.0;
    sum[1] = 0.0;
    ok = iarray_sum (subcube, sum);
    if (!ok)
    {
	fprintf (stderr, "Error computing sum\n");
	iarray_dealloc (subcube);
	return;
    }
    num = (sc.length.z * sc.length.y * sc.length.x);
    iarray_get_data_scaling (subcube, &scale, &offset);
    iarray_format_value (subcube, txt, sum[0], scale, offset * num);
    fprintf (stderr, "Region sum: %s\n", txt);
    iarray_dealloc (subcube);
}   /*  End Function sum_cbk   */
